@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,21 +6,50 @@
 
 #include <config.h>
 #include <asiolink/asio_wrapper.h>
+#include <asiolink/io_service_mgr.h>
 #include <cc/command_interpreter.h>
 #include <config/command_mgr.h>
-#include <d2/d2_log.h>
-#include <d2/d2_cfg_mgr.h>
 #include <d2/d2_controller.h>
 #include <d2/d2_process.h>
+#include <d2srv/d2_cfg_mgr.h>
+#include <d2srv/d2_log.h>
+#include <d2srv/d2_stats.h>
+#include <d2srv/d2_tsig_key.h>
+#include <hooks/hooks.h>
+#include <hooks/hooks_manager.h>
 
+using namespace isc::asiolink;
+using namespace isc::config;
+using namespace isc::hooks;
 using namespace isc::process;
 
+namespace {
+
+/// Structure that holds registered hook indexes.
+struct D2ProcessHooks {
+    int hooks_index_d2_srv_configured_;
+
+    /// Constructor that registers hook points for the D2 server.
+    D2ProcessHooks() {
+        hooks_index_d2_srv_configured_ = HooksManager::registerHook("d2_srv_configured");
+    }
+
+};
+
+// Declare a Hooks object. As this is outside any function or method, it
+// will be instantiated (and the constructor run) when the module is loaded.
+// As a result, the hook indexes will be defined before any method in this
+// module is called.
+D2ProcessHooks Hooks;
+
+}
+
 namespace isc {
 namespace d2 {
 
 // Setting to 80% for now. This is an arbitrary choice and should probably
 // be configurable.
-const unsigned int D2Process::QUEUE_RESTART_PERCENT =  80;
+const unsigned int D2Process::QUEUE_RESTART_PERCENT = 80;
 
 D2Process::D2Process(const char* name, const asiolink::IOServicePtr& io_service)
     : DProcessBase(name, io_service, DCfgMgrBasePtr(new D2CfgMgr())),
@@ -31,19 +60,22 @@ D2Process::D2Process(const char* name, const asiolink::IOServicePtr& io_service)
     // been received.  This means that until we receive the configuration,
     // D2 will neither receive nor process NameChangeRequests.
     // Pass in IOService for NCR IO event processing.
-    queue_mgr_.reset(new D2QueueMgr(getIoService()));
+    queue_mgr_.reset(new D2QueueMgr(getIOService()));
 
     // Instantiate update manager.
     // Pass in both queue manager and configuration manager.
     // Pass in IOService for DNS update transaction IO event processing.
     D2CfgMgrPtr tmp = getD2CfgMgr();
-    update_mgr_.reset(new D2UpdateMgr(queue_mgr_,  tmp,  getIoService()));
+    update_mgr_.reset(new D2UpdateMgr(queue_mgr_, tmp, getIOService()));
+
+    // Initialize stats manager.
+    D2Stats::init();
 };
 
 void
 D2Process::init() {
     // CommandMgr uses IO service to run asynchronous socket operations.
-    isc::config::CommandMgr::instance().setIOService(getIoService());
+    isc::config::CommandMgr::instance().setIOService(getIOService());
 };
 
 void
@@ -65,7 +97,7 @@ D2Process::run() {
             // process finished ones.
             update_mgr_->sweep();
 
-            // Wait on IO event(s)  - block until one or more of the following
+            // Wait on IO event(s) - block until one or more of the following
             // has occurred:
             //   a. NCR message has been received
             //   b. Transaction IO has completed
@@ -87,9 +119,9 @@ D2Process::run() {
                    "Process run method failed: " << ex.what());
     }
 
-    // @todo - if queue isn't empty, we may need to persist its contents
-    // this might be the place to do it, once there is a persistence mgr.
-    // This may also be better in checkQueueStatus.
+    /// @todo - if queue isn't empty, we may need to persist its contents
+    /// this might be the place to do it, once there is a persistence mgr.
+    /// This may also be better in checkQueueStatus.
 
     controller->deregisterCommands();
 
@@ -99,28 +131,27 @@ D2Process::run() {
 
 size_t
 D2Process::runIO() {
+    // Handle events registered by hooks using external IOService objects.
+    IOServiceMgr::instance().pollIOServices();
     // We want to block until at least one handler is called.  We'll use
     // boost::asio::io_service directly for two reasons. First off
-    // asiolink::IOService::run_one is a void and boost::asio::io_service::stopped
+    // asiolink::IOService::runOne is a void and boost::asio::io_service::stopped
     // is not present in older versions of boost.  We need to know if any
     // handlers ran or if the io_service was stopped.  That latter represents
     // some form of error and the application cannot proceed with a stopped
     // service.  Secondly, asiolink::IOService does not provide the poll
     // method.  This is a handy method which runs all ready handlers without
     // blocking.
-    asiolink::IOServicePtr& io = getIoService();
-    boost::asio::io_service& asio_io_service  = io->get_io_service();
 
     // Poll runs all that are ready. If none are ready it returns immediately
     // with a count of zero.
-    size_t cnt = asio_io_service.poll();
+    size_t cnt = getIOService()->poll();
     if (!cnt) {
         // Poll ran no handlers either none are ready or the service has been
-        // stopped.  Either way, call run_one to wait for a IO event. If the
+        // stopped.  Either way, call runOne to wait for a IO event. If the
         // service is stopped it will return immediately with a cnt of zero.
-        cnt = asio_io_service.run_one();
+        cnt = getIOService()->runOne();
     }
-
     return (cnt);
 }
 
@@ -192,27 +223,29 @@ D2Process::shutdown(isc::data::ConstElementPtr args) {
                 shutdown_type_ = SD_NOW;
             } else {
                 setShutdownFlag(false);
-                return (isc::config::createAnswer(1, "Invalid Shutdown type: "
-                                                  + type_str));
+                return (isc::config::createAnswer(CONTROL_RESULT_ERROR,
+                                                  "Invalid Shutdown type: " +
+                                                  type_str));
             }
         }
     }
 
     // Set the base class's shutdown flag.
     setShutdownFlag(true);
-    return (isc::config::createAnswer(0, "Shutdown initiated, type is: "
-                                      + type_str));
+    return (isc::config::createAnswer(CONTROL_RESULT_SUCCESS,
+                                      "Shutdown initiated, type is: " +
+                                      type_str));
 }
 
 isc::data::ConstElementPtr
 D2Process::configure(isc::data::ConstElementPtr config_set, bool check_only) {
     LOG_DEBUG(d2_logger, isc::log::DBGLVL_TRACE_BASIC, DHCP_DDNS_CONFIGURE)
         .arg(check_only ? "check" : "update")
-        .arg(config_set->str());
+        .arg(getD2CfgMgr()->redactConfig(config_set)->str());
 
     isc::data::ConstElementPtr answer;
     answer = getCfgMgr()->simpleParseConfig(config_set, check_only,
-                boost::bind(&D2Process::reconfigureCommandChannel, this));
+                std::bind(&D2Process::reconfigureCommandChannel, this));
     if (check_only) {
         return (answer);
     }
@@ -237,11 +270,51 @@ D2Process::configure(isc::data::ConstElementPtr config_set, bool check_only) {
     // the method we are in now is invoked as part of the configuration event
     // callback.  This means you can't wait for events here, you are already
     // in one.
-    // (@todo NOTE This could be turned into a bitmask of flags if we find other
-    // things that need reconfiguration.  It might also be useful if we
-    // did some analysis to decide what if anything we need to do.)
+    /// (@todo NOTE This could be turned into a bitmask of flags if we find other
+    /// things that need reconfiguration.  It might also be useful if we
+    /// did some analysis to decide what if anything we need to do.)
     reconf_queue_flag_ = true;
 
+    // This hook point notifies hooks libraries that the configuration of the
+    // D2 server has completed. It provides the hook library with the pointer
+    // to the common IO service object, new server configuration in the JSON
+    // format and with the pointer to the configuration storage where the
+    // parsed configuration is stored.
+    std::string error("");
+    if (HooksManager::calloutsPresent(Hooks.hooks_index_d2_srv_configured_)) {
+        CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
+
+        callout_handle->setArgument("io_context", getIOService());
+        callout_handle->setArgument("json_config", config_set);
+        callout_handle->setArgument("server_config",
+                                    getD2CfgMgr()->getD2CfgContext());
+        callout_handle->setArgument("error", error);
+
+        HooksManager::callCallouts(Hooks.hooks_index_d2_srv_configured_,
+                                   *callout_handle);
+
+        // The config can be rejected by a hook.
+        if (callout_handle->getStatus() == CalloutHandle::NEXT_STEP_DROP) {
+            callout_handle->getArgument("error", error);
+            LOG_ERROR(d2_logger, DHCP_DDNS_CONFIGURED_CALLOUT_DROP)
+                .arg(error);
+            reconf_queue_flag_ = false;
+            answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, error);
+            return (answer);
+        }
+    }
+
+    /// Let postponed hook initializations run.
+    try {
+        // Handle events registered by hooks using external IOService objects.
+        IOServiceMgr::instance().pollIOServices();
+    } catch (const std::exception& ex) {
+        std::ostringstream err;
+        err << "Error initializing hooks: "
+            << ex.what();
+        return (isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str()));
+    }
+
     // If we are here, configuration was valid, at least it parsed correctly
     // and therefore contained no invalid values.
     // Return the success answer from above.
@@ -253,10 +326,10 @@ D2Process::checkQueueStatus() {
     switch (queue_mgr_->getMgrState()){
     case D2QueueMgr::RUNNING:
         if (reconf_queue_flag_ || shouldShutdown()) {
-            // If we need to reconfigure the queue manager or we have been
-            // told to shutdown, then stop listening first.  Stopping entails
-            // canceling active listening which may generate an IO event, so
-            // instigate the stop and get out.
+            /// If we need to reconfigure the queue manager or we have been
+            /// told to shutdown, then stop listening first.  Stopping entails
+            /// canceling active listening which may generate an IO event, so
+            /// instigate the stop and get out.
             try {
                 LOG_DEBUG(d2_logger, isc::log::DBGLVL_START_SHUT,
                           DHCP_DDNS_QUEUE_MGR_STOPPING)
@@ -272,9 +345,9 @@ D2Process::checkQueueStatus() {
         break;
 
     case D2QueueMgr::STOPPED_QUEUE_FULL: {
-            // Resume receiving once the queue has decreased by twenty
-            // percent.  This is an arbitrary choice. @todo this value should
-            // probably be configurable.
+            /// Resume receiving once the queue has decreased by twenty
+            /// percent.  This is an arbitrary choice.
+            /// @todo this value should probably be configurable.
             size_t threshold = (((queue_mgr_->getMaxQueueSize()
                                 * QUEUE_RESTART_PERCENT)) / 100);
             if (queue_mgr_->getQueueSize() <= threshold) {
@@ -292,13 +365,13 @@ D2Process::checkQueueStatus() {
         }
 
     case D2QueueMgr::STOPPED_RECV_ERROR:
-        // If the receive error is not due to some fallout from shutting
-        // down then we will attempt to recover by reconfiguring the listener.
-        // This will close and destruct the current listener and make a new
-        // one with new resources.
-        // @todo This may need a safety valve such as retry count or a timer
-        // to keep from endlessly retrying over and over, with little time
-        // in between.
+        /// If the receive error is not due to some fallout from shutting
+        /// down then we will attempt to recover by reconfiguring the listener.
+        /// This will close and destruct the current listener and make a new
+        /// one with new resources.
+        /// @todo This may need a safety valve such as retry count or a timer
+        /// to keep from endlessly retrying over and over, with little time
+        /// in between.
         if (!shouldShutdown()) {
             LOG_INFO (d2_logger, DHCP_DDNS_QUEUE_MGR_RECOVERING);
             reconfigureQueueMgr();
@@ -306,10 +379,10 @@ D2Process::checkQueueStatus() {
         break;
 
     case D2QueueMgr::STOPPING:
-        // We are waiting for IO to cancel, so this is a NOP.
-        // @todo Possible timer for self-defense?  We could conceivably
-        // get into a condition where we never get the event, which would
-        // leave us stuck in stopping.  This is hugely unlikely but possible?
+        /// We are waiting for IO to cancel, so this is a NOP.
+        /// @todo Possible timer for self-defense?  We could conceivably
+        /// get into a condition where we never get the event, which would
+        /// leave us stuck in stopping.  This is hugely unlikely but possible?
         break;
 
     default:
@@ -333,9 +406,9 @@ D2Process::reconfigureQueueMgr() {
     // queue manager in INITTED state, which is fine.
     // What we don't want is to continually attempt to reconfigure so set
     // the flag false now.
-    // @todo This method assumes only 1 type of listener.  This will change
-    // to support at least a TCP version, possibly some form of RDBMS listener
-    // as well.
+    /// @todo This method assumes only 1 type of listener.  This will change
+    /// to support at least a TCP version, possibly some form of RDBMS listener
+    /// as well.
     reconf_queue_flag_ = false;
     try {
         // Wipe out the current listener.
@@ -344,11 +417,13 @@ D2Process::reconfigureQueueMgr() {
         // Get the configuration parameters that affect Queue Manager.
         const D2ParamsPtr& d2_params = getD2CfgMgr()->getD2Params();
 
-        // Warn the user if the server address is not the loopback.
+        /// Warn the user if the server address is not the loopback.
         /// @todo Remove this once we provide a secure mechanism.
-        std::string ip_address =  d2_params->getIpAddress().toText();
-        if (ip_address != "127.0.0.1" && ip_address != "::1") {
-            LOG_WARN(d2_logger, DHCP_DDNS_NOT_ON_LOOPBACK).arg(ip_address);
+        std::string ip_address = d2_params->getIpAddress().toText();
+        if (ip_address == "0.0.0.0" || ip_address == "::") {
+            LOG_WARN(d2_logger, DHCP_DDNS_LISTENING_ON_ALL_INTERFACES).arg(ip_address);
+        } else if (ip_address != "127.0.0.1" && ip_address != "::1") {
+           LOG_WARN(d2_logger, DHCP_DDNS_NOT_ON_LOOPBACK).arg(ip_address);
         }
 
         // Instantiate the listener.
@@ -365,8 +440,9 @@ D2Process::reconfigureQueueMgr() {
         }
 
         // Now start it. This assumes that starting is a synchronous,
-        // blocking call that executes quickly.  @todo Should that change then
-        // we will have to expand the state model to accommodate this.
+        // blocking call that executes quickly.
+        /// @todo Should that change then we will have to expand the state model
+        /// to accommodate this.
         queue_mgr_->startListening();
     } catch (const isc::Exception& ex) {
         // Queue manager failed to initialize and therefore not listening.
@@ -377,7 +453,10 @@ D2Process::reconfigureQueueMgr() {
 }
 
 D2Process::~D2Process() {
-};
+    queue_mgr_->stopListening();
+    getIOService()->stopAndPoll();
+    queue_mgr_->removeListener();
+}
 
 D2CfgMgrPtr
 D2Process::getD2CfgMgr() {
@@ -388,7 +467,7 @@ D2Process::getD2CfgMgr() {
 }
 
 const char* D2Process::getShutdownTypeStr(const ShutdownType& type) {
-    const char* str = "invalid";
+    const char* str;
     switch (type) {
     case SD_NORMAL:
         str = "normal";
@@ -400,6 +479,7 @@ const char* D2Process::getShutdownTypeStr(const ShutdownType& type) {
         str = "now";
         break;
     default:
+        str = "invalid";
         break;
     }
 
@@ -439,5 +519,5 @@ D2Process::reconfigureCommandChannel() {
     current_control_socket_ = sock_cfg;
 }
 
-}; // namespace isc::d2
-}; // namespace isc
+} // namespace isc::d2
+} // namespace isc
