@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -72,38 +72,6 @@ public:
     /// @param exit_value integer value to the process should exit with.
     virtual void shutdownServer(int exit_value);
 
-    /// @brief Command processor
-    ///
-    /// This method is uniform for all config backends. It processes received
-    /// command (as a string + JSON arguments). Internally, it's just a
-    /// wrapper that calls process*Command() methods and catches exceptions
-    /// in them.
-    ///
-    /// Currently supported commands are:
-    /// - shutdown
-    /// - libreload
-    /// - config-reload
-    /// - config-set
-    /// - config-get
-    /// - config-hash-get
-    /// - config-test
-    /// - dhcp-disable
-    /// - dhcp-enable
-    /// - version-get
-    /// - build-report
-    /// - leases-reclaim
-    /// - config-write
-    /// ...
-    ///
-    /// @note It never throws.
-    ///
-    /// @param command Text representation of the command (e.g. "shutdown")
-    /// @param args Optional parameters
-    ///
-    /// @return status of the command
-    static isc::data::ConstElementPtr
-    processCommand(const std::string& command, isc::data::ConstElementPtr args);
-
     /// @brief Configuration processor
     ///
     /// This is a method for handling incoming configuration updates.
@@ -168,19 +136,6 @@ private:
     commandShutdownHandler(const std::string& command,
                            isc::data::ConstElementPtr args);
 
-    /// @brief Handler for processing 'libreload' command
-    ///
-    /// This handler processes libreload command, which unloads all hook
-    /// libraries and reloads them.
-    ///
-    /// @param command (parameter ignored)
-    /// @param args (parameter ignored)
-    ///
-    /// @return status of the command
-    isc::data::ConstElementPtr
-    commandLibReloadHandler(const std::string& command,
-                            isc::data::ConstElementPtr args);
-
     /// @brief Handler for processing 'config-reload' command
     ///
     /// This handler processes config-reload command, which processes
