@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -273,9 +273,6 @@ public:
         /// Used while parsing shared-networks structures.
         SHARED_NETWORK,
 
-        /// Used while parsing Dhcp4/reservation-mode.
-        RESERVATION_MODE,
-
         /// Used while parsing Dhcp4/option-def structures.
         OPTION_DEF,
 
@@ -314,7 +311,7 @@ public:
         /// Used while parsing Dhcp4/loggers structures.
         LOGGERS,
 
-        /// Used while parsing Dhcp4/loggers/output_options structures.
+        /// Used while parsing Dhcp4/loggers/output-options structures.
         OUTPUT_OPTIONS,
 
         /// Used while parsing Dhcp4/dhcp-ddns.
@@ -338,6 +335,8 @@ public:
         /// Used while parsing compatibility parameters
         COMPATIBILITY,
 
+        /// Used while parsing Dhcp4/ib-ddns-conflict-resolution-mode.
+        DDNS_CONFLICT_RESOLUTION_MODE,
     } ParserContext;
 
     /// @brief File name
