@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,7 +9,7 @@
 #include <dhcp/option.h>
 #include <dhcp/option_int_array.h>
 #include <dhcp/option_vendor.h>
-#include <dhcp/tests/iface_mgr_test_config.h>
+#include <dhcp/testutils/iface_mgr_test_config.h>
 #include <dhcpsrv/lease.h>
 #include <dhcp4/tests/dhcp4_client.h>
 #include <util/multi_threading_mgr.h>
@@ -137,9 +137,8 @@ Dhcp4Client::appendPRL() {
         // has been specified to be requested.
         OptionUint8ArrayPtr prl(new OptionUint8Array(Option::V4,
                                   DHO_DHCP_PARAMETER_REQUEST_LIST));
-        for (std::set<uint8_t>::const_iterator opt = requested_options_.begin();
-             opt != requested_options_.end(); ++opt) {
-            prl->addValue(*opt);
+        for (auto const& opt : requested_options_) {
+            prl->addValue(opt);
         }
         context_.query_->addOption(prl);
     }
@@ -235,20 +234,18 @@ void
 Dhcp4Client::appendExtraOptions() {
     // If there are any custom options specified, add them all to the message.
     if (!extra_options_.empty()) {
-        for (OptionCollection::iterator opt = extra_options_.begin();
-             opt != extra_options_.end(); ++opt) {
+        for (auto const& opt : extra_options_) {
             // Call base class function so that unittests can add multiple
             // options with the same code.
-            context_.query_->Pkt::addOption(opt->second);
+            context_.query_->Pkt::addOption(opt.second);
         }
     }
 }
 
 void
 Dhcp4Client::appendClasses() {
-    for (ClientClasses::const_iterator cclass = classes_.cbegin();
-         cclass != classes_.cend(); ++cclass) {
-        context_.query_->addClass(*cclass);
+    for (auto const& cclass : classes_) {
+        context_.query_->addClass(cclass);
     }
 }
 
@@ -556,16 +553,15 @@ Dhcp4Client::sendMsg(const Pkt4Ptr& msg) {
     msg_copy->setIndex(iface_index_);
     // Copy classes
     const ClientClasses& classes = msg->getClasses();
-    for (ClientClasses::const_iterator cclass = classes.cbegin();
-         cclass != classes.cend(); ++cclass) {
-        msg_copy->addClass(*cclass);
+    for (auto const& cclass : classes) {
+        msg_copy->addClass(cclass);
     }
     srv_->fakeReceive(msg_copy);
 
     try {
-        // Invoke run_one instead of run, because we want to avoid triggering
+        // Invoke runOne instead of run, because we want to avoid triggering
         // IO service.
-        srv_->run_one();
+        srv_->runOne();
     } catch (...) {
         // Suppress errors, as the DHCPv4 server does.
     }
