@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,6 +6,8 @@
 
 #include <config.h>
 
+#include <asiolink/io_service_mgr.h>
+#include <asiolink/process_spawn.h>
 #include <cc/command_interpreter.h>
 #include <cc/data.h>
 #include <config/command_mgr.h>
@@ -26,7 +28,7 @@
 #include <hooks/hooks_manager.h>
 #include <process/cfgrpt/config_report.h>
 #include <stats/stats_mgr.h>
-#include <util/encode/hex.h>
+#include <util/encode/encode.h>
 #include <util/multi_threading_mgr.h>
 
 #include <signal.h>
@@ -78,11 +80,9 @@ static const char* SERVER_DUID_FILE = "kea-dhcp6-serverid";
 void signalHandler(int signo) {
     // SIGHUP signals a request to reconfigure the server.
     if (signo == SIGHUP) {
-        ControlledDhcpv6Srv::processCommand("config-reload",
-                                            ConstElementPtr());
+        CommandMgr::instance().processCommand(createCommand("config-reload"));
     } else if ((signo == SIGTERM) || (signo == SIGINT)) {
-        ControlledDhcpv6Srv::processCommand("shutdown",
-                                            ConstElementPtr());
+        CommandMgr::instance().processCommand(createCommand("shutdown"));
     }
 }
 
@@ -109,10 +109,6 @@ ControlledDhcpv6Srv::init(const std::string& file_name) {
         isc_throw(isc::BadValue, reason);
     }
 
-    // We don't need to call openActiveSockets() or startD2() as these
-    // methods are called in processConfig() which is called by
-    // processCommand("config-set", ...)
-
     // Set signal handlers. When the SIGHUP is received by the process
     // the server reconfiguration will be triggered. When SIGTERM or
     // SIGINT will be received, the server will start shutting down.
@@ -163,13 +159,13 @@ ControlledDhcpv6Srv::loadConfigFile(const std::string& file_name) {
         }
 
         // Use parsed JSON structures to configure the server
-        result = ControlledDhcpv6Srv::processCommand("config-set", json);
+        result = CommandMgr::instance().processCommand(createCommand("config-set", json));
         if (!result) {
             // Undetermined status of the configuration. This should never
             // happen, but as the configureDhcp6Server returns a pointer, it is
             // theoretically possible that it will return NULL.
             isc_throw(isc::BadValue, "undefined result of "
-                      "processCommand(\"config-set\", json)");
+                      "process command \"config-set\"");
         }
 
         // Now check is the returned result is successful (rcode=0) or not
@@ -229,42 +225,6 @@ ControlledDhcpv6Srv::commandShutdownHandler(const string&, ConstElementPtr args)
     return (createAnswer(CONTROL_RESULT_SUCCESS, "Shutting down."));
 }
 
-ConstElementPtr
-ControlledDhcpv6Srv::commandLibReloadHandler(const string&, ConstElementPtr) {
-    LOG_WARN(dhcp6_logger, DHCP6_DEPRECATED).arg("libreload command");
-
-    // stop thread pool (if running)
-    MultiThreadingCriticalSection cs;
-
-    // Clear the packet queue.
-    MultiThreadingMgr::instance().getThreadPool().reset();
-
-    try {
-        /// Get list of currently loaded libraries and reload them.
-        HookLibsCollection loaded = HooksManager::getLibraryInfo();
-        HooksManager::prepareUnloadLibraries();
-        static_cast<void>(HooksManager::unloadLibraries());
-        bool multi_threading_enabled = true;
-        uint32_t thread_count = 0;
-        uint32_t queue_size = 0;
-        CfgMultiThreading::extract(CfgMgr::instance().getStagingCfg()->getDHCPMultiThreading(),
-                                   multi_threading_enabled, thread_count, queue_size);
-        bool status = HooksManager::loadLibraries(loaded, multi_threading_enabled);
-        if (!status) {
-            isc_throw(Unexpected, "Failed to reload hooks libraries "
-                                  "(WARNING: libreload is deprecated).");
-        }
-    } catch (const std::exception& ex) {
-        LOG_ERROR(dhcp6_logger, DHCP6_HOOKS_LIBS_RELOAD_FAIL);
-        ConstElementPtr answer = isc::config::createAnswer(CONTROL_RESULT_ERROR, ex.what());
-        return (answer);
-    }
-    ConstElementPtr answer = isc::config::createAnswer(CONTROL_RESULT_SUCCESS,
-                             "Hooks libraries successfully reloaded "
-                             "(WARNING: libreload is deprecated).");
-    return (answer);
-}
-
 ConstElementPtr
 ControlledDhcpv6Srv::commandConfigReloadHandler(const string&,
                                                 ConstElementPtr /*args*/) {
@@ -382,7 +342,7 @@ ControlledDhcpv6Srv::commandConfigSetHandler(const string&,
 
     // Check unsupported objects.
     if (message.empty()) {
-        for (auto obj : args->mapValue()) {
+        for (auto const& obj : args->mapValue()) {
             const string& obj_name = obj.first;
             if (obj_name != "Dhcp6") {
                 LOG_ERROR(dhcp6_logger, DHCP6_CONFIG_UNSUPPORTED_OBJECT)
@@ -452,6 +412,17 @@ ControlledDhcpv6Srv::commandConfigSetHandler(const string&,
         LOG_FATAL(dhcp6_logger, DHCP6_CONFIG_UNRECOVERABLE_ERROR);
     }
 
+    /// Let postponed hook initializations run.
+    try {
+        // Handle events registered by hooks using external IOService objects.
+        IOServiceMgr::instance().pollIOServices();
+    } catch (const std::exception& ex) {
+        std::ostringstream err;
+        err << "Error initializing hooks: "
+            << ex.what();
+        return (isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str()));
+    }
+
     return (result);
 }
 
@@ -477,7 +448,7 @@ ControlledDhcpv6Srv::commandConfigTestHandler(const string&,
 
     // Check unsupported objects.
     if (message.empty()) {
-        for (auto obj : args->mapValue()) {
+        for (auto const& obj : args->mapValue()) {
             const string& obj_name = obj.first;
             if (obj_name != "Dhcp6") {
                 LOG_ERROR(dhcp6_logger, DHCP6_CONFIG_UNSUPPORTED_OBJECT)
@@ -522,7 +493,7 @@ ControlledDhcpv6Srv::commandDhcpDisableHandler(const std::string&,
 
     // If the args map does not contain 'origin' parameter, the default type
     // will be used (user command).
-    NetworkState::Origin type = NetworkState::Origin::USER_COMMAND;
+    auto type = NetworkState::USER_COMMAND;
 
     // Parse arguments to see if the 'max-period' or 'origin' parameters have
     // been specified.
@@ -547,17 +518,27 @@ ControlledDhcpv6Srv::commandDhcpDisableHandler(const std::string&,
                     }
                 }
             }
+            // 'origin-id' replaces the older parameter 'origin' since Kea 2.5.8
+            // stable release. However, the 'origin' is kept for backward compatibility
+            // with Kea versions before 2.5.8. It is common to receive both parameters
+            // because HA hook library sends both in case the partner server hasn't been
+            // upgraded to the new version. The 'origin-id' takes precedence over the
+            // 'origin'.
+            ConstElementPtr origin_id_element = args->get("origin-id");
             ConstElementPtr origin_element = args->get("origin");
-            // The 'origin' parameter is optional.
-            if (origin_element) {
-                // It must be a string, if specified.
-                if (origin_element->getType() != Element::string) {
-                    message << "'origin' argument must be a string";
-
+            // The 'origin-id' and 'origin' arguments are optional.
+            if (origin_id_element) {
+                if (origin_id_element->getType() == Element::integer) {
+                    type = origin_id_element->intValue();
                 } else {
+                    message << "'origin-id' argument must be a number";
+                }
+            } else if (origin_element) {
+                switch (origin_element->getType()) {
+                case Element::string:
                     origin = origin_element->stringValue();
                     if (origin == "ha-partner") {
-                        type = NetworkState::Origin::HA_COMMAND;
+                        type = NetworkState::HA_REMOTE_COMMAND;
                     } else if (origin != "user") {
                         if (origin.empty()) {
                             origin = "(empty string)";
@@ -565,6 +546,13 @@ ControlledDhcpv6Srv::commandDhcpDisableHandler(const std::string&,
                         message << "invalid value used for 'origin' parameter: "
                                 << origin;
                     }
+                    break;
+                case Element::integer:
+                    type = origin_element->intValue();
+                    break;
+                default:
+                    // It must be a string or a number, if specified.
+                    message << "'origin' argument must be a string or a number";
                 }
             }
         }
@@ -579,8 +567,8 @@ ControlledDhcpv6Srv::commandDhcpDisableHandler(const std::string&,
             // The user specified that the DHCP service should resume not
             // later than in max-period seconds. If the 'dhcp-enable' command
             // is not sent, the DHCP service will resume automatically.
-            network_state_->delayedEnableAll(static_cast<unsigned>(max_period),
-                                             type);
+            network_state_->delayedEnableService(static_cast<unsigned>(max_period),
+                                                 type);
         }
         network_state_->disableService(type);
 
@@ -600,7 +588,7 @@ ControlledDhcpv6Srv::commandDhcpEnableHandler(const std::string&,
 
     // If the args map does not contain 'origin' parameter, the default type
     // will be used (user command).
-    NetworkState::Origin type = NetworkState::Origin::USER_COMMAND;
+    auto type = NetworkState::USER_COMMAND;
 
     // Parse arguments to see if the 'origin' parameter has been specified.
     if (args) {
@@ -609,17 +597,27 @@ ControlledDhcpv6Srv::commandDhcpEnableHandler(const std::string&,
             message << "arguments for the 'dhcp-enable' command must be a map";
 
         } else {
+            // 'origin-id' replaces the older parameter 'origin' since Kea 2.5.8
+            // stable release. However, the 'origin' is kept for backward compatibility
+            // with Kea versions before 2.5.8. It is common to receive both parameters
+            // because HA hook library sends both in case the partner server hasn't been
+            // upgraded to the new version. The 'origin-id' takes precedence over the
+            // 'origin'.
+            ConstElementPtr origin_id_element = args->get("origin-id");
             ConstElementPtr origin_element = args->get("origin");
-            // The 'origin' parameter is optional.
-            if (origin_element) {
-                // It must be a string, if specified.
-                if (origin_element->getType() != Element::string) {
-                    message << "'origin' argument must be a string";
-
+            // The 'origin-id' and 'origin' arguments are optional.
+            if (origin_id_element) {
+                if (origin_id_element->getType() == Element::integer) {
+                    type = origin_id_element->intValue();
                 } else {
+                    message << "'origin-id' argument must be a number";
+                }
+            } else if (origin_element) {
+                switch (origin_element->getType()) {
+                case Element::string:
                     origin = origin_element->stringValue();
                     if (origin == "ha-partner") {
-                        type = NetworkState::Origin::HA_COMMAND;
+                        type = NetworkState::HA_REMOTE_COMMAND;
                     } else if (origin != "user") {
                         if (origin.empty()) {
                             origin = "(empty string)";
@@ -627,6 +625,13 @@ ControlledDhcpv6Srv::commandDhcpEnableHandler(const std::string&,
                         message << "invalid value used for 'origin' parameter: "
                                 << origin;
                     }
+                    break;
+                case Element::integer:
+                    type = origin_element->intValue();
+                    break;
+                default:
+                    // It must be a string or a number, if specified.
+                    message << "'origin' argument must be a string or a number";
                 }
             }
         }
@@ -772,6 +777,9 @@ ControlledDhcpv6Srv::commandStatusGetHandler(const string&,
         status->set("multi-threading-enabled", Element::create(false));
     }
 
+    status->set("extended-info-tables", Element::create(
+                    CfgMgr::instance().getCurrentCfg()->getCfgDbAccess()->getExtendedInfoTablesEnabled()));
+
     // Iterate through the interfaces and get all the errors.
     ElementPtr socket_errors(Element::createList());
     for (IfacePtr const& interface : IfaceMgr::instance().getIfaces()) {
@@ -824,87 +832,14 @@ ControlledDhcpv6Srv::commandStatisticSetMaxSampleAgeAllHandler(const string&,
     return (answer);
 }
 
-ConstElementPtr
-ControlledDhcpv6Srv::processCommand(const string& command,
-                                    ConstElementPtr args) {
-    string txt = args ? args->str() : "(none)";
-
-    LOG_DEBUG(dhcp6_logger, DBG_DHCP6_COMMAND, DHCP6_COMMAND_RECEIVED)
-              .arg(command).arg(txt);
-
-    ControlledDhcpv6Srv* srv = ControlledDhcpv6Srv::getInstance();
-
-    if (!srv) {
-        ConstElementPtr no_srv = isc::config::createAnswer(CONTROL_RESULT_ERROR,
-            "Server object not initialized, so can't process command '" +
-            command + "', arguments: '" + txt + "'.");
-        return (no_srv);
-    }
-
-    try {
-        if (command == "shutdown") {
-            return (srv->commandShutdownHandler(command, args));
-
-        } else if (command == "libreload") {
-            return (srv->commandLibReloadHandler(command, args));
-
-        } else if (command == "config-reload") {
-            return (srv->commandConfigReloadHandler(command, args));
-
-        } else if (command == "config-set") {
-            return (srv->commandConfigSetHandler(command, args));
-
-        } else if (command == "config-get") {
-            return (srv->commandConfigGetHandler(command, args));
-
-        } else if (command == "config-hash-get") {
-            return (srv->commandConfigHashGetHandler(command, args));
-
-        } else if (command == "config-test") {
-            return (srv->commandConfigTestHandler(command, args));
-
-        } else if (command == "dhcp-disable") {
-            return (srv->commandDhcpDisableHandler(command, args));
-
-        } else if (command == "dhcp-enable") {
-            return (srv->commandDhcpEnableHandler(command, args));
-
-        } else if (command == "version-get") {
-            return (srv->commandVersionGetHandler(command, args));
-
-        } else if (command == "build-report") {
-            return (srv->commandBuildReportHandler(command, args));
-
-        } else if (command == "leases-reclaim") {
-            return (srv->commandLeasesReclaimHandler(command, args));
-
-        } else if (command == "config-write") {
-            return (srv->commandConfigWriteHandler(command, args));
-
-        } else if (command == "server-tag-get") {
-            return (srv->commandServerTagGetHandler(command, args));
-
-        } else if (command == "config-backend-pull") {
-            return (srv->commandConfigBackendPullHandler(command, args));
-
-        } else if (command == "status-get") {
-            return (srv->commandStatusGetHandler(command, args));
-        }
-
-        return (isc::config::createAnswer(CONTROL_RESULT_ERROR, "Unrecognized command:"
-                                          + command));
-
-    } catch (const isc::Exception& ex) {
-        return (isc::config::createAnswer(CONTROL_RESULT_ERROR, "Error while processing command '"
-                                          + command + "': " + ex.what() +
-                                          ", params: '" + txt + "'"));
-    }
-}
-
 isc::data::ConstElementPtr
 ControlledDhcpv6Srv::processConfig(isc::data::ConstElementPtr config) {
     ControlledDhcpv6Srv* srv = ControlledDhcpv6Srv::getInstance();
 
+    // Allow DB reconnect on startup. The database connection parameters specify
+    // respective details.
+    DbConnectionInitWithRetry retry;
+
     // Single stream instance used in all error clauses
     std::ostringstream err;
 
@@ -950,7 +885,7 @@ ControlledDhcpv6Srv::processConfig(isc::data::ConstElementPtr config) {
         cfg_db->setAppendedParameters(params);
         cfg_db->createManagers();
         // Reset counters related to connections as all managers have been recreated.
-        srv->getNetworkState()->reset(NetworkState::Origin::DB_CONNECTION);
+        srv->getNetworkState()->resetForDbConnection();
     } catch (const std::exception& ex) {
         err << "Unable to open database: " << ex.what();
         return (isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str()));
@@ -1080,7 +1015,9 @@ ControlledDhcpv6Srv::processConfig(isc::data::ConstElementPtr config) {
         CfgMgr::instance().getStagingCfg()->getCfgSubnets6()->initAllocatorsAfterConfigure();
 
     } catch (const std::exception& ex) {
-        err << "Error initializing the lease allocators: " << ex.what();
+        err << "Error initializing the lease allocators: "
+            << ex.what();
+        return (isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str()));
     }
 
     // Apply multi threading settings.
@@ -1157,17 +1094,17 @@ ControlledDhcpv6Srv::ControlledDhcpv6Srv(uint16_t server_port /*= DHCP6_SERVER_P
     }
     server_ = this; // remember this instance for later use in handlers
 
+    // ProcessSpawn uses IO service to handle signal set events.
+    ProcessSpawn::setIOService(getIOService());
+
     // TimerMgr uses IO service to run asynchronous timers.
     TimerMgr::instance()->setIOService(getIOService());
 
     // CommandMgr uses IO service to run asynchronous socket operations.
     CommandMgr::instance().setIOService(getIOService());
 
-    // LeaseMgr uses IO service to run asynchronous timers.
-    LeaseMgr::setIOService(getIOService());
-
-    // HostMgr uses IO service to run asynchronous timers.
-    HostMgr::setIOService(getIOService());
+    // DatabaseConnection uses IO service to run asynchronous timers.
+    DatabaseConnection::setIOService(getIOService());
 
     // These are the commands always supported by the DHCPv6 server.
     // Please keep the list in alphabetic order.
@@ -1201,9 +1138,6 @@ ControlledDhcpv6Srv::ControlledDhcpv6Srv(uint16_t server_port /*= DHCP6_SERVER_P
     CommandMgr::instance().registerCommand("dhcp-disable",
         std::bind(&ControlledDhcpv6Srv::commandDhcpDisableHandler, this, ph::_1, ph::_2));
 
-    CommandMgr::instance().registerCommand("libreload",
-        std::bind(&ControlledDhcpv6Srv::commandLibReloadHandler, this, ph::_1, ph::_2));
-
     CommandMgr::instance().registerCommand("leases-reclaim",
         std::bind(&ControlledDhcpv6Srv::commandLeasesReclaimHandler, this, ph::_1, ph::_2));
 
@@ -1262,7 +1196,6 @@ ControlledDhcpv6Srv::~ControlledDhcpv6Srv() {
         MultiThreadingMgr::instance().apply(false, 0, 0);
         LeaseMgrFactory::destroy();
         HostMgr::create();
-        cleanup();
 
         // The closure captures either a shared pointer (memory leak)
         // or a raw pointer (pointing to a deleted object).
@@ -1272,6 +1205,8 @@ ControlledDhcpv6Srv::~ControlledDhcpv6Srv() {
 
         timer_mgr_->unregisterTimers();
 
+        cleanup();
+
         // Close the command socket (if it exists).
         CommandMgr::instance().closeCommandSocket();
 
@@ -1287,7 +1222,6 @@ ControlledDhcpv6Srv::~ControlledDhcpv6Srv() {
         CommandMgr::instance().deregisterCommand("dhcp-disable");
         CommandMgr::instance().deregisterCommand("dhcp-enable");
         CommandMgr::instance().deregisterCommand("leases-reclaim");
-        CommandMgr::instance().deregisterCommand("libreload");
         CommandMgr::instance().deregisterCommand("server-tag-get");
         CommandMgr::instance().deregisterCommand("shutdown");
         CommandMgr::instance().deregisterCommand("statistic-get");
@@ -1303,15 +1237,11 @@ ControlledDhcpv6Srv::~ControlledDhcpv6Srv() {
         CommandMgr::instance().deregisterCommand("status-get");
         CommandMgr::instance().deregisterCommand("version-get");
 
-        // LeaseMgr uses IO service to run asynchronous timers.
-        LeaseMgr::setIOService(IOServicePtr());
-
-        // HostMgr uses IO service to run asynchronous timers.
-        HostMgr::setIOService(IOServicePtr());
+        // Reset DatabaseConnection IO service.
+        DatabaseConnection::setIOService(IOServicePtr());
     } catch (...) {
         // Don't want to throw exceptions from the destructor. The server
         // is shutting down anyway.
-        ;
     }
 
     server_ = NULL; // forget this instance. There should be no callback anymore
@@ -1353,7 +1283,7 @@ ControlledDhcpv6Srv::dbLostCallback(ReconnectCtlPtr db_reconnect_ctl) {
     // Disable service until the connection is recovered.
     if (db_reconnect_ctl->retriesLeft() == db_reconnect_ctl->maxRetries() &&
         db_reconnect_ctl->alterServiceState()) {
-        network_state_->disableService(NetworkState::Origin::DB_CONNECTION);
+        network_state_->disableService(NetworkState::DB_CONNECTION);
     }
 
     LOG_INFO(dhcp6_logger, DHCP6_DB_RECONNECT_LOST_CONNECTION);
@@ -1384,7 +1314,7 @@ ControlledDhcpv6Srv::dbRecoveredCallback(ReconnectCtlPtr db_reconnect_ctl) {
 
     // Enable service after the connection is recovered.
     if (db_reconnect_ctl->alterServiceState()) {
-        network_state_->enableService(NetworkState::Origin::DB_CONNECTION);
+        network_state_->enableService(NetworkState::DB_CONNECTION);
     }
 
     LOG_INFO(dhcp6_logger, DHCP6_DB_RECONNECT_SUCCEEDED);
