@@ -8,6 +8,7 @@
 namespace isc {
 namespace dhcp {
 
+extern const isc::log::MessageID DHCP4_HOOK_SUBNET6_SELECT_PARKING_LOT_FULL;
 extern const isc::log::MessageID DHCP6_ACTIVATE_INTERFACE;
 extern const isc::log::MessageID DHCP6_ADD_GLOBAL_STATUS_CODE;
 extern const isc::log::MessageID DHCP6_ADD_STATUS_CODE_FOR_IA;
@@ -18,11 +19,12 @@ extern const isc::log::MessageID DHCP6_BUFFER_WAIT_SIGNAL;
 extern const isc::log::MessageID DHCP6_CB_ON_DEMAND_FETCH_UPDATES_FAIL;
 extern const isc::log::MessageID DHCP6_CB_PERIODIC_FETCH_UPDATES_FAIL;
 extern const isc::log::MessageID DHCP6_CB_PERIODIC_FETCH_UPDATES_RETRIES_EXHAUSTED;
+extern const isc::log::MessageID DHCP6_CLASSES_ASSIGNED;
+extern const isc::log::MessageID DHCP6_CLASSES_ASSIGNED_AFTER_SUBNET_SELECTION;
 extern const isc::log::MessageID DHCP6_CLASS_ASSIGNED;
 extern const isc::log::MessageID DHCP6_CLASS_UNCONFIGURED;
 extern const isc::log::MessageID DHCP6_CLASS_UNDEFINED;
 extern const isc::log::MessageID DHCP6_CLASS_UNTESTABLE;
-extern const isc::log::MessageID DHCP6_COMMAND_RECEIVED;
 extern const isc::log::MessageID DHCP6_CONFIG_COMPLETE;
 extern const isc::log::MessageID DHCP6_CONFIG_LOAD_FAIL;
 extern const isc::log::MessageID DHCP6_CONFIG_PACKET_QUEUE;
@@ -55,18 +57,17 @@ extern const isc::log::MessageID DHCP6_DECLINE_FAIL_LEASE_WITHOUT_DUID;
 extern const isc::log::MessageID DHCP6_DECLINE_FAIL_NO_LEASE;
 extern const isc::log::MessageID DHCP6_DECLINE_LEASE;
 extern const isc::log::MessageID DHCP6_DECLINE_PROCESS_IA;
-extern const isc::log::MessageID DHCP6_DEPRECATED;
 extern const isc::log::MessageID DHCP6_DEVELOPMENT_VERSION;
 extern const isc::log::MessageID DHCP6_DHCP4O6_PACKET_RECEIVED;
 extern const isc::log::MessageID DHCP6_DHCP4O6_RECEIVE_FAIL;
 extern const isc::log::MessageID DHCP6_DHCP4O6_RECEIVING;
+extern const isc::log::MessageID DHCP6_DHCP4O6_RESPONSE_DATA;
 extern const isc::log::MessageID DHCP6_DHCP4O6_SEND_FAIL;
 extern const isc::log::MessageID DHCP6_DYNAMIC_RECONFIGURATION;
 extern const isc::log::MessageID DHCP6_DYNAMIC_RECONFIGURATION_FAIL;
 extern const isc::log::MessageID DHCP6_DYNAMIC_RECONFIGURATION_SUCCESS;
 extern const isc::log::MessageID DHCP6_FLEX_ID;
 extern const isc::log::MessageID DHCP6_HANDLE_SIGNAL_EXCEPTION;
-extern const isc::log::MessageID DHCP6_HOOKS_LIBS_RELOAD_FAIL;
 extern const isc::log::MessageID DHCP6_HOOK_BUFFER_RCVD_DROP;
 extern const isc::log::MessageID DHCP6_HOOK_BUFFER_RCVD_SKIP;
 extern const isc::log::MessageID DHCP6_HOOK_BUFFER_SEND_SKIP;
@@ -82,6 +83,7 @@ extern const isc::log::MessageID DHCP6_HOOK_PACKET_RCVD_SKIP;
 extern const isc::log::MessageID DHCP6_HOOK_PACKET_SEND_DROP;
 extern const isc::log::MessageID DHCP6_HOOK_PACKET_SEND_SKIP;
 extern const isc::log::MessageID DHCP6_HOOK_SUBNET6_SELECT_DROP;
+extern const isc::log::MessageID DHCP6_HOOK_SUBNET6_SELECT_PARK;
 extern const isc::log::MessageID DHCP6_HOOK_SUBNET6_SELECT_SKIP;
 extern const isc::log::MessageID DHCP6_INIT_FAIL;
 extern const isc::log::MessageID DHCP6_LEASE_ADVERT;
@@ -111,8 +113,10 @@ extern const isc::log::MessageID DHCP6_PACKET_DROP_SERVERID_MISMATCH;
 extern const isc::log::MessageID DHCP6_PACKET_DROP_UNICAST;
 extern const isc::log::MessageID DHCP6_PACKET_OPTIONS_SKIPPED;
 extern const isc::log::MessageID DHCP6_PACKET_PROCESS_EXCEPTION;
+extern const isc::log::MessageID DHCP6_PACKET_PROCESS_EXCEPTION_MAIN;
 extern const isc::log::MessageID DHCP6_PACKET_PROCESS_FAIL;
 extern const isc::log::MessageID DHCP6_PACKET_PROCESS_STD_EXCEPTION;
+extern const isc::log::MessageID DHCP6_PACKET_PROCESS_STD_EXCEPTION_MAIN;
 extern const isc::log::MessageID DHCP6_PACKET_QUEUE_FULL;
 extern const isc::log::MessageID DHCP6_PACKET_RECEIVED;
 extern const isc::log::MessageID DHCP6_PACKET_RECEIVE_FAIL;
@@ -132,9 +136,12 @@ extern const isc::log::MessageID DHCP6_PD_LEASE_REUSE;
 extern const isc::log::MessageID DHCP6_PROCESS_IA_NA_EXTEND;
 extern const isc::log::MessageID DHCP6_PROCESS_IA_NA_RELEASE;
 extern const isc::log::MessageID DHCP6_PROCESS_IA_NA_REQUEST;
+extern const isc::log::MessageID DHCP6_PROCESS_IA_NA_SOLICIT;
 extern const isc::log::MessageID DHCP6_PROCESS_IA_PD_EXTEND;
 extern const isc::log::MessageID DHCP6_PROCESS_IA_PD_REQUEST;
+extern const isc::log::MessageID DHCP6_PROCESS_IA_PD_SOLICIT;
 extern const isc::log::MessageID DHCP6_QUERY_DATA;
+extern const isc::log::MessageID DHCP6_QUERY_LABEL;
 extern const isc::log::MessageID DHCP6_RAPID_COMMIT;
 extern const isc::log::MessageID DHCP6_RECLAIM_EXPIRED_LEASES_FAIL;
 extern const isc::log::MessageID DHCP6_RELEASE_NA;
