@@ -1,18 +1,21 @@
-// Copyright (C) 2013-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-/// @file   dhcp6_test_utils.h
+/// @file dhcp6_test_utils.h
 ///
-/// @brief  This file contains utility classes used for DHCPv6 server testing
+/// @brief This file contains utility classes used for DHCPv6 server testing
 
 #ifndef DHCP6_TEST_UTILS_H
 #define DHCP6_TEST_UTILS_H
 
 #include <gtest/gtest.h>
 
+#include <asiolink/process_spawn.h>
+#include <dhcp6/dhcp6_srv.h>
+#include <dhcp6/parser_context.h>
 #include <dhcp/pkt6.h>
 #include <dhcp/option6_ia.h>
 #include <dhcp/option6_iaaddr.h>
@@ -22,13 +25,13 @@
 #include <dhcp/option_custom.h>
 #include <dhcp/option.h>
 #include <dhcp/iface_mgr.h>
-#include <dhcp/tests/iface_mgr_test_config.h>
+#include <dhcp/testutils/iface_mgr_test_config.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/lease_mgr.h>
 #include <dhcpsrv/lease_mgr_factory.h>
-#include <dhcp6/dhcp6_srv.h>
-#include <dhcp6/parser_context.h>
 #include <hooks/hooks_manager.h>
+#include <config/command_mgr.h>
+#include <util/multi_threading_mgr.h>
 
 #include <list>
 
@@ -67,7 +70,6 @@ struct SpecializedTypeWrapper {
     ValueType value_;
 };
 
-
 /// @brief Class representing strongly typed IAID.
 struct IAID : public SpecializedTypeWrapper<uint32_t> {
     /// @brief Constructor
@@ -79,7 +81,7 @@ struct IAID : public SpecializedTypeWrapper<uint32_t> {
 
 /// @brief Class representing strongly typed value for strict IAID checks.
 ///
-/// Strict IAID checks are used to verify that  the particular address has been
+/// Strict IAID checks are used to verify that the particular address has been
 /// assign to a specific IA. In many cases we don't check that because it may
 /// not be possible to predict to which IA the specific lease will be assigned.
 struct StrictIAIDChecking : public SpecializedTypeWrapper<bool> {
@@ -105,7 +107,6 @@ struct StrictIAIDChecking : public SpecializedTypeWrapper<bool> {
     }
 };
 
-
 /// @brief Base class for DHCPv6 server testing.
 ///
 /// Currently it configures the test data path directory in
@@ -127,7 +128,6 @@ private:
 
     /// @brief Holds the original data directory.
     std::string original_datadir_;
-
 };
 
 /// @brief "naked" Dhcpv6Srv class that exposes internal members
@@ -137,6 +137,14 @@ public:
         // Open the "memfile" database for leases
         std::string memfile = "type=memfile universe=6 persist=false";
         isc::dhcp::LeaseMgrFactory::create(memfile);
+
+        isc::asiolink::ProcessSpawn::setIOService(getIOService());
+
+        db::DatabaseConnection::setIOService(getIOService());
+
+        dhcp::TimerMgr::instance()->setIOService(getIOService());
+
+        config::CommandMgr::instance().setIOService(getIOService());
     }
 
     /// @brief fakes packet reception
@@ -148,7 +156,6 @@ public:
     ///
     /// See fake_received_ field for description
     virtual isc::dhcp::Pkt6Ptr receivePacket(int /*timeout*/) {
-
         // If there is anything prepared as fake incoming
         // traffic, use it
         if (!fake_received_.empty()) {
@@ -157,6 +164,9 @@ public:
             return (pkt);
         }
 
+        // Make sure the server processed all packets in MT.
+        isc::util::MultiThreadingMgr::instance().getThreadPool().wait(3);
+
         // If not, just trigger shutdown and
         // return immediately
         shutdown();
@@ -169,29 +179,80 @@ public:
     /// it in fake_send_ list where test can later inspect
     /// server's response.
     virtual void sendPacket(const isc::dhcp::Pkt6Ptr& pkt) {
+        isc::util::MultiThreadingLock lock(mutex_);
         fake_sent_.push_back(pkt);
     }
 
+    /// @brief fake receive packet from server
+    ///
+    /// The client uses this packet as a reply from the server.
+    ///
+    /// @return The received packet.
+    Pkt6Ptr receiveOneMsg() {
+        // Make sure the server processed all packets.
+        isc::util::MultiThreadingMgr::instance().getThreadPool().wait(2);
+
+        // Lock the mutex for the rest of the function.
+        isc::util::MultiThreadingLock lock(mutex_);
+
+        // Return empty pointer if server hasn't responded.
+        if (fake_sent_.empty()) {
+            return (Pkt6Ptr());
+        }
+
+        Pkt6Ptr msg = fake_sent_.front();
+        fake_sent_.pop_front();
+        return (msg);
+    }
+
     /// @brief adds a packet to fake receive queue
     ///
     /// See fake_received_ field for description
     void fakeReceive(const isc::dhcp::Pkt6Ptr& pkt) {
+        // Add packet events normally set by PktFilter.
+        pkt->addPktEvent(PktEvent::SOCKET_RECEIVED);
+        pkt->addPktEvent(PktEvent::BUFFER_READ);
         fake_received_.push_back(pkt);
     }
 
-    virtual ~NakedDhcpv6Srv() {
-        // Close the lease database
-        isc::dhcp::LeaseMgrFactory::destroy();
+    virtual ~NakedDhcpv6Srv() = default;
+
+    /// @brief Processes incoming Solicit message.
+    ///
+    /// @param request a message received from client
+    /// @return REPLY message or null
+    Pkt6Ptr processSolicit(const Pkt6Ptr& solicit) {
+        AllocEngine::ClientContext6 ctx;
+        bool drop = !earlyGHRLookup(solicit, ctx);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        ctx.subnet_ = selectSubnet(solicit, drop);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        initContext(ctx, drop);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        return (processSolicit(ctx));
     }
 
     /// @brief Processes incoming Request message.
     ///
     /// @param request a message received from client
-    /// @return REPLY message or NULL
+    /// @return REPLY message or null
     Pkt6Ptr processRequest(const Pkt6Ptr& request) {
         AllocEngine::ClientContext6 ctx;
-        bool drop = false;
-        initContext(request, ctx, drop);
+        bool drop = !earlyGHRLookup(request, ctx);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        ctx.subnet_ = selectSubnet(request, drop);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        initContext(ctx, drop);
         if (drop) {
             return (Pkt6Ptr());
         }
@@ -201,11 +262,18 @@ public:
     /// @brief Processes incoming Renew message.
     ///
     /// @param renew a message received from client
-    /// @return REPLY message or NULL
+    /// @return REPLY message or null
     Pkt6Ptr processRenew(const Pkt6Ptr& renew) {
         AllocEngine::ClientContext6 ctx;
-        bool drop = false;
-        initContext(renew, ctx, drop);
+        bool drop = !earlyGHRLookup(renew, ctx);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        ctx.subnet_ = selectSubnet(renew, drop);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        initContext(ctx, drop);
         if (drop) {
             return (Pkt6Ptr());
         }
@@ -215,11 +283,18 @@ public:
     /// @brief Processes incoming Rebind message.
     ///
     /// @param rebind a message received from client
-    /// @return REPLY message or NULL
+    /// @return REPLY message or null
     Pkt6Ptr processRebind(const Pkt6Ptr& rebind) {
         AllocEngine::ClientContext6 ctx;
-        bool drop = false;
-        initContext(rebind, ctx, drop);
+        bool drop = !earlyGHRLookup(rebind, ctx);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        ctx.subnet_ = selectSubnet(rebind, drop);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        initContext(ctx, drop);
         if (drop) {
             return (Pkt6Ptr());
         }
@@ -229,11 +304,18 @@ public:
     /// @brief Processes incoming Release message.
     ///
     /// @param release a message received from client
-    /// @return REPLY message or NULL
+    /// @return REPLY message or null
     Pkt6Ptr processRelease(const Pkt6Ptr& release) {
         AllocEngine::ClientContext6 ctx;
-        bool drop = false;
-        initContext(release, ctx, drop);
+        bool drop = !earlyGHRLookup(release, ctx);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        ctx.subnet_ = selectSubnet(release, drop);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        initContext(ctx, drop);
         if (drop) {
             return (Pkt6Ptr());
         }
@@ -243,11 +325,18 @@ public:
     /// @brief Processes incoming Decline message.
     ///
     /// @param decline a message received from client
-    /// @return REPLY message or NULL
+    /// @return REPLY message or null
     Pkt6Ptr processDecline(const Pkt6Ptr& decline) {
         AllocEngine::ClientContext6 ctx;
-        bool drop = false;
-        initContext(decline, ctx, drop);
+        bool drop = !earlyGHRLookup(decline, ctx);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        ctx.subnet_ = selectSubnet(decline, drop);
+        if (drop) {
+            return (Pkt6Ptr());
+        }
+        initContext(ctx, drop);
         if (drop) {
             return (Pkt6Ptr());
         }
@@ -272,11 +361,12 @@ public:
     using Dhcpv6Srv::shutdown_;
     using Dhcpv6Srv::name_change_reqs_;
     using Dhcpv6Srv::VENDOR_CLASS_PREFIX;
+    using Dhcpv6Srv::earlyGHRLookup;
     using Dhcpv6Srv::initContext;
     using Dhcpv6Srv::server_port_;
     using Dhcpv6Srv::client_port_;
 
-    /// @brief packets we pretend to receive
+    /// @brief packets we pretend to receive.
     ///
     /// Instead of setting up sockets on interfaces that change between
     /// OSes, it is much easier to fake packet reception. This is a list
@@ -285,7 +375,11 @@ public:
     /// NakedDhcpv6Srv::receivePacket() methods.
     std::list<isc::dhcp::Pkt6Ptr> fake_received_;
 
+    /// @brief packets we pretend to send.
     std::list<isc::dhcp::Pkt6Ptr> fake_sent_;
+
+    /// @brief Mutex to protect the packet buffers.
+    std::mutex mutex_;
 };
 
 /// @brief Test fixture for any tests requiring blank/empty configuration
@@ -296,9 +390,17 @@ public:
     /// @brief Constructor
     NakedDhcpv6SrvTest();
 
-    // Generate IA_NA or IA_PD option with specified parameters
-    boost::shared_ptr<isc::dhcp::Option6IA> generateIA
-        (uint16_t type, uint32_t iaid, uint32_t t1, uint32_t t2);
+    /// @brief Generate IA_NA or IA_PD option with specified parameters
+    ///
+    /// @param type The option type (usually 4 for IA_NA, 25 for IA_PD)
+    /// @param iaid The identity association identifier (id of IA)
+    /// @param t1 The T1 time
+    /// @param t2 The t2 time
+    /// @return The generated option
+    boost::shared_ptr<isc::dhcp::Option6IA> generateIA(uint16_t type,
+                                                       uint32_t iaid,
+                                                       uint32_t t1,
+                                                       uint32_t t2);
 
     /// @brief generates interface-id option, based on text
     ///
@@ -307,22 +409,22 @@ public:
     /// @return pointer to the option object
     isc::dhcp::OptionPtr generateInterfaceId(const std::string& iface_id) {
         isc::dhcp::OptionBuffer tmp(iface_id.begin(), iface_id.end());
-        return (isc::dhcp::OptionPtr
-                (new isc::dhcp::Option(isc::dhcp::Option::V6,
-                                       D6O_INTERFACE_ID, tmp)));
+        return (isc::dhcp::OptionPtr(new isc::dhcp::Option(isc::dhcp::Option::V6,
+                                                           D6O_INTERFACE_ID, tmp)));
     }
 
     /// @brief Generate binary data option
     ///
     /// Creates an Option in the V6 space with the given type and binary data
-    /// of the given number of bytes.  The data is initialized to the values
+    /// of the given number of bytes. The data is initialized to the values
     /// 100 to 100 + n, where n is the desired number of bytes.
     ///
     /// @param type option type for the new option
     /// @param data_size number of bytes of data to generate
     ///
     /// @return Pointer to the new option
-    isc::dhcp::OptionPtr generateBinaryOption(const DHCPv6OptionType type, size_t data_size) {
+    isc::dhcp::OptionPtr generateBinaryOption(const DHCPv6OptionType type,
+                                              size_t data_size) {
         if (data_size == 0) {
             return (isc::dhcp::OptionPtr
                     (new isc::dhcp::Option(isc::dhcp::Option::V6, type)));
@@ -358,8 +460,7 @@ public:
     // Checks if server response (ADVERTISE or REPLY) includes proper
     // server-id.
     void checkServerId(const isc::dhcp::Pkt6Ptr& rsp,
-                       const isc::dhcp::OptionPtr& expected_srvid)
-    {
+                       const isc::dhcp::OptionPtr& expected_srvid) {
         // check that server included its server-id
         isc::dhcp::OptionPtr tmp = rsp->getOption(D6O_SERVERID);
         EXPECT_EQ(tmp->getType(), expected_srvid->getType() );
@@ -370,8 +471,7 @@ public:
     // Checks if server response (ADVERTISE or REPLY) includes proper
     // client-id.
     void checkClientId(const isc::dhcp::Pkt6Ptr& rsp,
-                       const isc::dhcp::OptionPtr& expected_clientid)
-    {
+                       const isc::dhcp::OptionPtr& expected_clientid) {
         // check that server included our own client-id
         isc::dhcp::OptionPtr tmp = rsp->getOption(D6O_CLIENTID);
         ASSERT_TRUE(tmp);
@@ -387,8 +487,7 @@ public:
                           uint8_t expected_message_type,
                           uint32_t expected_transid,
                           uint16_t expected_status_code,
-                          uint32_t expected_t1, uint32_t expected_t2)
-    {
+                          uint32_t expected_t1, uint32_t expected_t2) {
         // Check if we get response at all
         checkResponse(rsp, expected_message_type, expected_transid);
 
@@ -421,14 +520,14 @@ public:
     /// @param expected_t1 expected T1 in IA_NA option
     /// @param expected_t2 expected T2 in IA_NA option
     /// @param check_addr whether to check for address with 0 lifetimes
-    void checkIA_NAStatusCode
-        (const boost::shared_ptr<isc::dhcp::Option6IA>& ia,
-         uint16_t expected_status_code, uint32_t expected_t1,
-         uint32_t expected_t2, bool check_addr = true);
+    void checkIA_NAStatusCode(const boost::shared_ptr<isc::dhcp::Option6IA>& ia,
+                              uint16_t expected_status_code,
+                              uint32_t expected_t1,
+                              uint32_t expected_t2,
+                              bool check_addr = true);
 
     void checkMsgStatusCode(const isc::dhcp::Pkt6Ptr& msg,
-                            uint16_t expected_status)
-    {
+                            uint16_t expected_status) {
         isc::dhcp::Option6StatusCodePtr status =
             boost::dynamic_pointer_cast<isc::dhcp::Option6StatusCode>
                 (msg->getOption(D6O_STATUS_CODE));
@@ -459,6 +558,14 @@ public:
         EXPECT_EQ(expected_transid, rsp->getTransid());
     }
 
+    /// @brief Checks the contents of a packet's event stack agains a list
+    /// of expected events.
+    ///
+    /// @param msg pointer to the packet under test.
+    /// @param expected_events a list of the event labels in the order they
+    /// are expected to occur in the stack.
+    void checkPktEvents(const PktPtr& msg, std::list<std::string> expected_events);
+
     virtual ~NakedDhcpv6SrvTest();
 
     // A DUID used in most tests (typically as client-id)
@@ -471,7 +578,10 @@ public:
     std::string valid_iface_;
 
     // Index of a valid network interface
-    uint32_t valid_ifindex_;
+    unsigned int valid_ifindex_;
+
+    /// @brief Time the test started (UTC/microseconds)
+    boost::posix_time::ptime start_time_;
 };
 
 // We need to pass one reference to the Dhcp6Client, which is defined in
@@ -497,29 +607,84 @@ public:
         NO_IA       // Client will not send IA_NA at all
     };
 
+    /// @brief Specifies if lease affinity is enabled or disabled
+    enum LeaseAffinity {
+        LEASE_AFFINITY_ENABLED,
+        LEASE_AFFINITY_DISABLED
+    };
+
+    class Dhcpv6SrvMTTestGuard {
+    public:
+        Dhcpv6SrvMTTestGuard(Dhcpv6SrvTest& test, bool mt_enabled) : test_(test) {
+            test_.setMultiThreading(mt_enabled);
+        }
+        ~Dhcpv6SrvMTTestGuard() {
+            test_.setMultiThreading(false);
+        }
+        Dhcpv6SrvTest& test_;
+    };
+
     /// @brief Constructor that initializes a simple default configuration
     ///
     /// Sets up a single subnet6 with one pool for addresses and second
     /// pool for prefixes.
     Dhcpv6SrvTest();
 
-    /// @brief destructor
+    /// @brief Destructor
     ///
     /// Removes existing configuration.
-    ~Dhcpv6SrvTest() {
-        isc::dhcp::CfgMgr::instance().clear();
-    };
+    virtual ~Dhcpv6SrvTest();
+
+    /// @brief Used to configure a server for tests.
+    ///
+    /// A wrapper over configureDhcp6Server() to which any other
+    /// simulations of production code are added.
+    ///
+    /// @brief server the server being tested
+    /// @brief config the configuration the server is configured with
+    ///
+    /// @return a JSON-formatted status of the reconfiguration
+    static ConstElementPtr configure(Dhcpv6Srv& server, isc::data::ConstElementPtr config);
 
     /// @brief Runs DHCPv6 configuration from the JSON string.
     ///
     /// @param config String holding server configuration in JSON format.
-    void configure(const std::string& config);
+    /// @param commit A boolean flag indicating if the new configuration
+    /// should be committed (if true), or not (if false).
+    /// @param open_sockets A boolean flag indicating if sockets should
+    /// be opened (if true), or not (if false).
+    /// @param create_managers A boolean flag indicating if managers should be
+    /// recreated.
+    /// @param test A boolean flag which indicates if only testing config.
+    /// @param lease_affinity A flag which indicates if lease affinity should
+    /// be enabled or disabled.
+    void configure(const std::string& config,
+                   const bool commit = true,
+                   const bool open_sockets = false,
+                   const bool create_managers = true,
+                   const bool test = false,
+                   const LeaseAffinity lease_affinity = LEASE_AFFINITY_DISABLED);
 
     /// @brief Configure the DHCPv6 server using the JSON string.
     ///
     /// @param config String holding server configuration in JSON format.
     /// @param srv Server to be configured.
-    void configure(const std::string& config, NakedDhcpv6Srv& srv);
+    /// @param commit A boolean flag indicating if the new configuration
+    /// should be committed (if true), or not (if false).
+    /// @param open_sockets A boolean flag indicating if sockets should
+    /// be opened (if true), or not (if false).
+    /// @param create_managers A boolean flag indicating if managers should be
+    /// recreated.
+    /// @param test A boolean flag which indicates if only testing config.
+    /// @param lease_affinity A flag which indicates if lease affinity should
+    /// be enabled or disabled.
+    void configure(const std::string& config,
+                   NakedDhcpv6Srv& srv,
+                   const bool commit = true,
+                   const bool open_sockets = false,
+                   const bool create_managers = true,
+                   const bool test = false,
+                   const LeaseAffinity lease_affinity = LEASE_AFFINITY_DISABLED);
 
     /// @brief Checks that server response (ADVERTISE or REPLY) contains proper
     ///        IA_NA option
@@ -545,7 +710,6 @@ public:
     checkIA_PD(const isc::dhcp::Pkt6Ptr& rsp, uint32_t expected_iaid,
                uint32_t expected_t1, uint32_t expected_t2);
 
-
     // Check that generated IAADDR option contains expected address
     // and lifetime values match the configured subnet
     /// @param expected_pref check that lease preferedlifetime has the not-zero
@@ -558,7 +722,6 @@ public:
                      uint32_t expected_pref = 0,
                      uint32_t expected_valid = 0) {
 
-
         // Check that the assigned address is indeed from the configured pool.
         // Note that when comparing addresses, we compare the textual
         // representation. IOAddress does not support being streamed to
@@ -587,9 +750,10 @@ public:
 
     // Checks if the lease sent to client is present in the database
     // and is valid when checked against the configured subnet
-    isc::dhcp::Lease6Ptr checkLease
-        (const isc::dhcp::DuidPtr& duid, const isc::dhcp::OptionPtr& ia_na,
-         boost::shared_ptr<isc::dhcp::Option6IAAddr> addr);
+    isc::dhcp::Lease6Ptr
+    checkLease(const isc::dhcp::DuidPtr& duid,
+               const isc::dhcp::OptionPtr& ia_na,
+               boost::shared_ptr<isc::dhcp::Option6IAAddr> addr);
 
     /// @brief Check if the specified lease is present in the data base.
     ///
@@ -606,9 +770,10 @@ public:
     /// @param ia_pd IA_PD option that contains the IAPRefix option
     /// @param prefix pointer to the IAPREFIX option
     /// @return corresponding IPv6 lease (if found)
-    isc::dhcp::Lease6Ptr checkPdLease
-      (const isc::dhcp::DuidPtr& duid, const isc::dhcp::OptionPtr& ia_pd,
-       boost::shared_ptr<isc::dhcp::Option6IAPrefix> prefix);
+    isc::dhcp::Lease6Ptr
+    checkPdLease(const isc::dhcp::DuidPtr& duid,
+                 const isc::dhcp::OptionPtr& ia_pd,
+                 boost::shared_ptr<isc::dhcp::Option6IAPrefix> prefix);
 
     /// @brief Creates a message with specified IA
     ///
@@ -653,8 +818,8 @@ public:
     /// two different instances of an option that has identical content
     /// will return true.
     ///
-    /// It is safe to pass NULL pointers. Two NULL pointers are equal.
-    /// NULL pointer and non-NULL pointers are obviously non-equal.
+    /// It is safe to pass null pointers. Two null pointers are equal.
+    /// null pointer and non-null pointers are obviously non-equal.
     ///
     /// @param option1 pointer to the first option
     /// @param option2
@@ -674,13 +839,16 @@ public:
     /// @param expected_result SHOULD_PASS if the lease is expected to
     /// be successfully declined, or SHOULD_FAIL if the lease is expected
     /// to not be declined.
+    /// @param config_index specifies which config index should be used:
+    /// 0 - memfile, 1 - mysql, 2 - pgsql
     void acquireAndDecline(Dhcp6Client& client,
                            const std::string& duid1,
                            const uint32_t iaid1,
                            const std::string& duid2,
                            const uint32_t iaid2,
                            AddressInclusion addr_type,
-                           ExpectedResult expected_result);
+                           ExpectedResult expected_result,
+                           uint8_t config_index = 0);
 
     /// @brief Performs basic (positive) RENEW test
     ///
@@ -736,15 +904,31 @@ public:
     /// @param type type (TYPE_NA or TYPE_PD)
     /// @param existing address to be preinserted into the database
     /// @param release_addr address being sent in RELEASE
+    /// @param lease_affinity A flag which indicates if lease affinity should
+    /// be enabled or disabled.
     void
     testReleaseBasic(isc::dhcp::Lease::Type type,
                      const isc::asiolink::IOAddress& existing,
-                     const isc::asiolink::IOAddress& release_addr);
+                     const isc::asiolink::IOAddress& release_addr,
+                     const LeaseAffinity lease_affinity);
+
+    /// @brief Checks that reassignment of a released-expired lease
+    /// does not lead to zero lifetimes.
+    ///
+    /// This method does not throw, but uses gtest macros to signify failures.
+    ///
+    /// @param type lease type (TYPE_NA or TYPE_PD).
+    /// @param addr lease address.
+    /// @param qtype query type.
+    void
+    testReleaseNoDelete(isc::dhcp::Lease::Type type,
+                        const isc::asiolink::IOAddress& addr,
+                        uint8_t qtype);
 
     /// @brief Performs negative RELEASE test
     ///
     /// See releaseReject and pdReleaseReject tests for detailed
-    /// explanation.  In essence the test attempts to perform couple
+    /// explanation. In essence the test attempts to perform couple
     /// failed RELEASE scenarios.
     ///
     /// This method does not throw, but uses gtest macros to signify failures.
@@ -761,18 +945,33 @@ public:
     /// @param stat_name this statistic is expected to be set to 1
     void testReceiveStats(uint8_t pkt_type, const std::string& stat_name);
 
+    /// @brief Set multi-threading mode.
+    void setMultiThreading(bool enabled) {
+        multi_threading_ = enabled;
+    }
 
-    /// A subnet used in most tests
+    /// @brief Check if example files contain valid configuration.
+    void checkConfigFiles();
+
+    /// @brief Check if the server configuration stored in file is valid.
+    ///
+    /// @param path The path to the configuration file.
+    void loadConfigFile(const std::string& path);
+
+    /// A subnet used in most tests.
     isc::dhcp::Subnet6Ptr subnet_;
 
-    /// A normal, non-temporary pool used in most tests
+    /// A normal, non-temporary pool used in most tests.
     isc::dhcp::Pool6Ptr pool_;
 
-    /// A prefix pool used in most tests
+    /// A prefix pool used in most tests.
     isc::dhcp::Pool6Ptr pd_pool_;
 
     /// @brief Server object under test.
     NakedDhcpv6Srv srv_;
+
+    /// @brief The multi-threading flag.
+    bool multi_threading_;
 };
 
 /// @brief Patch the server config to add interface-config/re-detect=false
@@ -787,13 +986,35 @@ disableIfacesReDetect(isc::data::ConstElementPtr json) {
     }
 }
 
+/// @brief Patch the server config to add multi-threading/enable-multi-threading
+/// @param json the server config
+inline void
+configureMultiThreading(bool enabled, isc::data::ConstElementPtr json) {
+    isc::data::ConstElementPtr multi_threading = json->get("multi-threading");
+    if (!multi_threading) {
+        isc::data::ElementPtr mutable_cfg =
+                boost::const_pointer_cast<isc::data::Element>(json);
+        multi_threading = isc::data::Element::createMap();
+        mutable_cfg->set("multi-threading", multi_threading);
+    }
+
+    isc::data::ElementPtr mutable_cfg =
+        boost::const_pointer_cast<isc::data::Element>(multi_threading);
+    if (enabled) {
+        mutable_cfg->set("enable-multi-threading", isc::data::Element::create(true));
+        mutable_cfg->set("thread-pool-size", isc::data::Element::create(4));
+        mutable_cfg->set("packet-queue-size", isc::data::Element::create(4));
+    } else {
+        mutable_cfg->set("enable-multi-threading", isc::data::Element::create(false));
+    }
+}
+
 /// @brief Runs parser in JSON mode, useful for parser testing
 ///
 /// @param in string to be parsed
 /// @return ElementPtr structure representing parsed JSON
 inline isc::data::ElementPtr
-parseJSON(const std::string& in)
-{
+parseJSON(const std::string& in) {
     isc::dhcp::Parser6Context ctx;
     return (ctx.parseString(in, isc::dhcp::Parser6Context::PARSER_JSON));
 }
@@ -807,8 +1028,7 @@ parseJSON(const std::string& in)
 /// @param verbose display the exception message when it fails
 /// @return ElementPtr structure representing parsed JSON
 inline isc::data::ElementPtr
-parseDHCP6(const std::string& in, bool verbose = false)
-{
+parseDHCP6(const std::string& in, bool verbose = false) {
     try {
         isc::dhcp::Parser6Context ctx;
         isc::data::ElementPtr json;
@@ -832,8 +1052,7 @@ parseDHCP6(const std::string& in, bool verbose = false)
 /// @param verbose display the exception message when it fails
 /// @return ElementPtr structure representing parsed JSON
 inline isc::data::ElementPtr
-parseOPTION_DEFS(const std::string& in, bool verbose = false)
-{
+parseOPTION_DEFS(const std::string& in, bool verbose = false) {
     try {
         isc::dhcp::Parser6Context ctx;
         return (ctx.parseString(in, isc::dhcp::Parser6Context::PARSER_OPTION_DEFS));
@@ -846,8 +1065,8 @@ parseOPTION_DEFS(const std::string& in, bool verbose = false)
     }
 }
 
-}; // end of isc::dhcp::test namespace
-}; // end of isc::dhcp namespace
-}; // end of isc namespace
+} // end of isc::dhcp::test namespace
+} // end of isc::dhcp namespace
+} // end of isc namespace
 
 #endif // DHCP6_TEST_UTILS_H
