@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -188,8 +188,7 @@ public:
 
 class JSONFileBackendTest : public dhcp::test::BaseServerTest {
 public:
-    JSONFileBackendTest()
-        : BaseServerTest() {
+    JSONFileBackendTest() {
     }
 
     ~JSONFileBackendTest() {
@@ -216,7 +215,7 @@ public:
     /// stops the IO service and causes the function to return.
     void runTimersWithTimeout(const IOServicePtr& io_service, const long timeout_ms,
                               std::function<bool()> cond = std::function<bool()>()) {
-        IntervalTimer timer(*io_service);
+        IntervalTimer timer(io_service);
         std::atomic<bool> stopped(false);
         timer.setup([&io_service, &stopped]() {
             stopped = true;
@@ -226,9 +225,10 @@ public:
         // Run as long as the timeout hasn't occurred and the interrupting
         // condition is not specified or not met.
         while (!stopped && (!cond || !cond())) {
-            io_service->run_one();
+            io_service->runOne();
         }
-        io_service->get_io_service().reset();
+        io_service->stop();
+        io_service->restart();
     }
 
     /// @brief This test verifies that the timer used to fetch the configuration
@@ -297,9 +297,8 @@ public:
             EXPECT_EQ(cb_control->getDatabaseCurrentConfigFetchCalls(), 0);
             EXPECT_EQ(cb_control->getDatabaseStagingConfigFetchCalls(), 1);
 
-            ConstElementPtr result =
-                ControlledDhcpv6Srv::processCommand("config-backend-pull",
-                                                    ConstElementPtr());
+            ConstElementPtr list_cmds = createCommand("config-backend-pull");
+            ConstElementPtr result = CommandMgr::instance().processCommand(list_cmds);
             EXPECT_EQ(cb_control->getDatabaseTotalConfigFetchCalls(), 2);
             std::string expected;
 
