@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -65,7 +65,7 @@ NetconfCfgMgr::getConfigSummary(const uint32_t /*selection*/) {
     ostringstream s;
 
     // Then print managed servers.
-    for (auto serv : *ctx->getCfgServersMap()) {
+    for (auto const& serv : *ctx->getCfgServersMap()) {
         if (s.tellp() != 0) {
             s << " ";
         }
@@ -156,7 +156,7 @@ NetconfConfig::toElement() const {
     netconf->set("hooks-libraries", hooks_config_.toElement());
     // Set managed-servers
     ElementPtr servers = Element::createMap();
-    for (auto serv : *servers_map_) {
+    for (auto const& serv : *servers_map_) {
         ElementPtr server = serv.second->toElement();
         servers->set(serv.first, server);
     }
