@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,10 +6,10 @@
 
 #include <config.h>
 
-#include <cfgrpt/config_report.h>
 #include <netconf/netconf_controller.h>
 #include <netconf/netconf_process.h>
 #include <netconf/parser_context.h>
+#include <process/cfgrpt/config_report.h>
 
 #include <signal.h>
 
@@ -54,7 +54,7 @@ NetconfController::createProcess() {
 }
 
 isc::data::ConstElementPtr
-NetconfController::parseFile(const std::string& name) {
+NetconfController::parseFile(string const& name) {
     ParserContext parser;
     return (parser.parseFile(name, ParserContext::PARSER_NETCONF));
 }
@@ -68,8 +68,5 @@ NetconfController::getNetconfProcess() {
     return (boost::dynamic_pointer_cast<NetconfProcess>(getProcess()));
 }
 
-// Refer to config_report so it will be embedded in the binary.
-const char* const* netconf_config_report = isc::detail::config_report;
-
-} // namespace isc::netconf
-} // namespace isc
+}  // namespace netconf
+}  // namespace isc
