@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -121,11 +121,6 @@ public:
             thread_->join();
             thread_.reset();
         }
-        // io_service must be stopped after the thread returns,
-        // otherwise the thread may never return if it is
-        // waiting for the completion of some asynchronous tasks.
-        io_service_->stop();
-        io_service_.reset();
         if (agent_) {
             clearYang(agent_);
             agent_->clear();
@@ -135,6 +130,7 @@ public:
         responses_.clear();
         removeUnixSocketFile();
         SysrepoSetup::cleanSharedMemory();
+        io_service_->stopAndPoll();
     }
 
     /// @brief Returns socket file path.
@@ -146,9 +142,9 @@ public:
         string socket_path;
         const char* env = getenv("KEA_SOCKET_TEST_DIR");
         if (env) {
-            socket_path = string(env) + "/test-socket";
+            socket_path = string(env) + "/" + TEST_SOCKET;
         } else {
-            socket_path = sandbox.join("test-socket");
+            socket_path = sandbox.join(TEST_SOCKET);
         }
         return (socket_path);
     }
@@ -193,7 +189,7 @@ public:
         : finished_(false),
           io_service_(new IOService()),
           thread_(),
-          agent_(new NakedNetconfAgent) {
+          agent_(new NakedNetconfAgent()) {
     }
 
     /// @brief Destructor.
@@ -259,7 +255,7 @@ void
 NetconfAgentTest::fakeServer() {
     // Acceptor.
     boost::asio::local::stream_protocol::acceptor
-        acceptor(io_service_->get_io_service());
+        acceptor(io_service_->getInternalIOService());
     EXPECT_NO_THROW_LOG(acceptor.open());
     boost::asio::local::stream_protocol::endpoint
         endpoint(unixSocketFilePath());
@@ -268,14 +264,14 @@ NetconfAgentTest::fakeServer() {
     EXPECT_NO_THROW_LOG(acceptor.bind(endpoint));
     EXPECT_NO_THROW_LOG(acceptor.listen());
     boost::asio::local::stream_protocol::socket
-        socket(io_service_->get_io_service());
+        socket(io_service_->getInternalIOService());
 
     // Ready.
     signalReady();
 
     // Timeout.
     bool timeout = false;
-    IntervalTimer timer(*io_service_);
+    IntervalTimer timer(io_service_);
     timer.setup([&timeout]() {
             timeout = true;
             FAIL() << "timeout";
@@ -291,7 +287,7 @@ NetconfAgentTest::fakeServer() {
                               accepted = true;
                           });
     while (!accepted && !timeout) {
-        io_service_->run_one();
+        io_service_->runOne();
     }
     ASSERT_FALSE(ec);
 
@@ -305,7 +301,7 @@ NetconfAgentTest::fakeServer() {
                              received = cnt;
                          });
     while (!received && !timeout) {
-        io_service_->run_one();
+        io_service_->runOne();
     }
     ASSERT_FALSE(ec);
     rbuf.resize(received);
@@ -334,7 +330,7 @@ NetconfAgentTest::fakeServer() {
                           sent = cnt;
                       });
     while (!sent && !timeout) {
-        io_service_->run_one();
+        io_service_->runOne();
     }
     ASSERT_FALSE(ec);
 
