@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -106,6 +106,14 @@ public:
     /// @throw BadValue if unsupported state value was provided.
     void setPartnerState(const std::string& state);
 
+    /// @brief Sets partner state unavailable.
+    ///
+    /// This function should be called whenever there is a problem to
+    /// communicate with the partner. Besides setting the state, it also
+    /// resets some other communication states. In particular, it resets
+    /// the clock skew.
+    void setPartnerUnavailable();
+
 private:
     /// @brief Sets partner state.
     ///
@@ -115,6 +123,22 @@ private:
     void setPartnerStateInternal(const std::string& state);
 
 public:
+
+    /// @brief Returns the duration since the partner was first seen in the
+    /// current state.
+    ///
+    /// @return A duration since the partner has been in the current state.
+    boost::posix_time::time_duration getDurationSincePartnerStateTime() const;
+
+private:
+
+    /// @brief Sets partner's state time to current time.
+    ///
+    /// It marks the time when the partner was first seen in the current state.
+    void setCurrentPartnerStateTimeInternal();
+
+public:
+
     /// @brief Returns scopes served by the partner server.
     ///
     /// @return A set of scopes served by the partner.
@@ -192,6 +216,19 @@ public:
     /// @return true if communication is interrupted, false otherwise.
     bool isCommunicationInterrupted() const;
 
+protected:
+
+    /// @brief Convenience function attempting to retrieve client
+    /// identifier from the DHCP message.
+    ///
+    /// @param message DHCPv4 or DHCPv6 message.
+    /// @param option_type code of the option holding the client id.
+    /// @return vector containing the client identifier or an empty
+    /// vector if it does not exist.
+    static std::vector<uint8_t> getClientId(const dhcp::PktPtr& message,
+                                            const uint16_t option_type);
+public:
+
     /// @brief Checks if the DHCP message appears to be unanswered.
     ///
     /// This method is used to provide the communication state with a
@@ -284,6 +321,118 @@ protected:
     /// See @c CommunicationState::analyzeMessage for details.
     virtual void clearConnectingClients() = 0;
 
+public:
+
+    /// @brief Returns the number of lease updates rejected by the partner (MT safe).
+    ///
+    /// Each rejected lease update is counted only once if it failed
+    /// multiple times. Before returning the counter, it discards expired
+    /// rejected lease updates.
+    ///
+    /// @return Current rejected lease update number count.
+    size_t getRejectedLeaseUpdatesCount();
+
+protected:
+
+    /// @brief Returns the number of lease updates rejected by the partner.
+    ///
+    /// Each rejected lease update is counted only once if it failed
+    /// multiple times. Before returning the counter, it discards expired
+    /// rejected lease updates.
+    ///
+    /// @return Current rejected lease update number count.
+    virtual size_t getRejectedLeaseUpdatesCountInternal() = 0;
+
+    /// @brief Extracts the number of lease updates rejected by the partner
+    /// from the specified container.
+    ///
+    /// @param rejected_clients container holding rejected clients (v4 or v6).
+    /// @tparam RejectedClientsType type of the container holding rejected
+    /// clients.
+    /// @return Current rejected lease update number count.
+    template<typename RejectedClientsType>
+    static size_t getRejectedLeaseUpdatesCountFromContainer(RejectedClientsType& rejected_clients) {
+        if (rejected_clients.empty()) {
+            return (0);
+        }
+        auto& idx = rejected_clients.template get<1>();
+        auto upper_limit = idx.upper_bound(time(NULL));
+        if (upper_limit != idx.end()) {
+            auto lower_limit = idx.cbegin();
+            idx.erase(lower_limit, upper_limit);
+        }
+        return (rejected_clients.size());
+    }
+
+public:
+
+    /// @brief Marks that the lease update failed due to a conflict for the
+    /// specified DHCP message (MT safe).
+    ///
+    /// If the conflict has been already reported for the given client, the
+    /// rejected lease count remains unchanged.
+    ///
+    /// @param message DHCP message for which a lease update failed due to
+    ///  a conflict.
+    /// @param lifetime a time in seconds after which the rejected lease
+    /// update entry should be discarded.
+    /// @return true if the update was rejected for the first time, false
+    /// otherwise.
+    bool reportRejectedLeaseUpdate(const dhcp::PktPtr& message,
+                                   const uint32_t lifetime = 86400);
+
+protected:
+
+    /// @brief Marks that the lease update failed due to a conflict for the
+    /// specified DHCP message.
+    ///
+    /// If the conflict has been already reported for the given client, the
+    /// rejected lease count remains unchanged.
+    ///
+    /// @param message DHCP message for which a lease update failed due to
+    ///  a conflict.
+    /// @param lifetime a time in seconds after which the rejected lease
+    /// update entry should be discarded.
+    /// @return true if the update was rejected for the first time, false
+    /// otherwise.
+    virtual bool reportRejectedLeaseUpdateInternal(const dhcp::PktPtr& message,
+                                                   const uint32_t lifetime) = 0;
+public:
+
+    /// @brief Marks the lease update successful (MT safe).
+    ///
+    /// If the lease update was previously marked "in conflict", it is
+    /// now cleared, effectively lowering the number of conflicted leases.
+    ///
+    /// @param message DHCP message for which the lease update was
+    /// successful.
+    /// @return true when the lease was marked "in conflict" and it is
+    /// now cleared.
+    bool reportSuccessfulLeaseUpdate(const dhcp::PktPtr& message);
+
+protected:
+
+    /// @brief Marks the lease update successful.
+    ///
+    /// If the lease update was previously marked "in conflict", it is
+    /// now cleared, effectively lowering the number of conflicted leases.
+    ///
+    /// @param message DHCP message for which the lease update was
+    /// successful.
+    /// @return true when the lease was marked "in conflict" and it is
+    /// now cleared.
+    virtual bool reportSuccessfulLeaseUpdateInternal(const dhcp::PktPtr& message) = 0;
+
+public:
+
+    /// @brief Clears rejected client leases (MT safe).
+    void clearRejectedLeaseUpdates();
+
+protected:
+
+    /// @brief Clears rejected client leases.
+    virtual void clearRejectedLeaseUpdatesInternal() = 0;
+
 public:
 
     /// @brief Issues a warning about high clock skew between the active
@@ -352,7 +501,7 @@ public:
     /// 60 seconds.  In the future it may become configurable.
     ///
     /// @return true if the HA service should enter "terminated" state.
-    bool clockSkewShouldTerminate() const;
+    bool clockSkewShouldTerminate();
 
 private:
     /// @brief Indicates whether the HA service should enter "terminated"
@@ -371,7 +520,7 @@ private:
     /// 60 seconds.  In the future it may become configurable.
     ///
     /// @return true if the HA service should enter "terminated" state.
-    bool clockSkewShouldTerminateInternal() const;
+    bool clockSkewShouldTerminateInternal();
 
     /// @brief Checks if the clock skew is greater than the specified number
     /// of seconds.
@@ -381,6 +530,28 @@ private:
     /// number of seconds, false otherwise.
     bool isClockSkewGreater(const long seconds) const;
 
+public:
+
+    /// @brief Indicates whether the HA service should enter "terminated"
+    /// state due to excessive number of rejected lease updates.
+    ///
+    /// @return true if the number of rejected lease updates is equal or
+    /// exceeds the value of max-rejected-lease-updates, false when the
+    /// max-rejected-lease-updates is 0 or is greater than the current
+    /// number of rejected lease updates.
+    bool rejectedLeaseUpdatesShouldTerminate();
+
+private:
+
+    /// @brief Indicates whether the HA service should enter "terminated"
+    /// state due to excessive number of rejected lease updates.
+    ///
+    /// @return true if the number of rejected lease updates is equal or
+    /// exceeds the value of max-rejected-lease-updates, false when the
+    /// max-rejected-lease-updates is 0 or is greater than the current
+    /// number of rejected lease updates.
+    bool rejectedLeaseUpdatesShouldTerminateInternal();
+
 public:
 
     /// @brief Provide partner's notion of time so the new clock skew can be
@@ -397,6 +568,7 @@ public:
     void setPartnerTime(const std::string& time_text);
 
 private:
+
     /// @brief Provide partner's notion of time so the new clock skew can be
     /// calculated.
     ///
@@ -410,6 +582,12 @@ private:
     /// precision.
     void setPartnerTimeInternal(const std::string& time_text);
 
+    /// @brief Resets the partner time and the clock skew to defaults.
+    ///
+    /// This function is called internally when the communication with the
+    /// partner fails.
+    void resetPartnerTimeInternal();
+
 public:
     /// @brief Returns current clock skew value in the logger friendly format.
     std::string logFormatClockSkew() const;
@@ -544,6 +722,9 @@ protected:
     /// Negative value means that the partner's state is unknown.
     int partner_state_;
 
+    /// Holds a time when partner was first seen in the current state.
+    boost::posix_time::ptime partner_state_time_;
+
     /// @brief Last known set of scopes served by the partner server.
     std::set<std::string> partner_scopes_;
 
@@ -649,6 +830,44 @@ public:
 
 protected:
 
+    /// @brief Returns the number of lease updates rejected by the partner.
+    ///
+    /// Each rejected lease update is counted only once if it failed
+    /// multiple times. Before returning the counter, it discards expired
+    /// rejected lease updates.
+    ///
+    /// @return Current rejected lease update number count.
+    virtual size_t getRejectedLeaseUpdatesCountInternal();
+
+    /// @brief Marks that the lease update failed due to a conflict for the
+    /// specified DHCP message.
+    ///
+    /// If the conflict has been already reported for the given client, the
+    /// rejected lease count remains unchanged.
+    ///
+    /// @param message DHCP message for which a lease update failed due to
+    ///  a conflict.
+    /// @param lifetime a time in seconds after which the rejected lease
+    /// update entry should be discarded.
+    /// @return true if the update was rejected for the first time, false
+    /// otherwise.
+    virtual bool reportRejectedLeaseUpdateInternal(const dhcp::PktPtr& message,
+                                                   const uint32_t lifetime);
+
+    /// @brief Marks the lease update successful.
+    ///
+    /// If the lease update was previously marked "in conflict", it is
+    /// now cleared, effectively lowering the number of conflicted leases.
+    ///
+    /// @param message DHCP message for which the lease update was
+    /// successful.
+    /// @return true when the lease was marked "in conflict" and it is
+    /// now cleared.
+    virtual bool reportSuccessfulLeaseUpdateInternal(const dhcp::PktPtr& message);
+
+    /// @brief Clears rejected client leases.
+    virtual void clearRejectedLeaseUpdatesInternal();
+
     /// @brief Checks if the DHCPv4 message appears to be unanswered.
     ///
     /// Should be called in a thread safe context.
@@ -683,8 +902,23 @@ protected:
     /// See @c CommunicationState::analyzeMessage for details.
     virtual void clearConnectingClients();
 
+    /// @brief Hashed index used in the multi index containers to find
+    /// clients by HW address and client identifier.
+    ///
+    /// @tparam ClientData Type of a structure holding client information.
+    template<typename ClientData>
+    using ClientIdent4 = boost::multi_index::hashed_unique<
+        boost::multi_index::composite_key<
+            ClientData,
+            boost::multi_index::member<ClientData, std::vector<uint8_t>,
+                                       &ClientData::hwaddr_>,
+            boost::multi_index::member<ClientData, std::vector<uint8_t>,
+                                       &ClientData::clientid_>
+            >
+        >;
+
     /// @brief Structure holding information about the client which has
-    /// send the packet being analyzed.
+    /// sent the packet being analyzed.
     struct ConnectingClient4 {
         std::vector<uint8_t> hwaddr_;
         std::vector<uint8_t> clientid_;
@@ -698,15 +932,7 @@ protected:
         boost::multi_index::indexed_by<
             // First index is a composite index which allows to find a client
             // by the HW address/client identifier tuple.
-            boost::multi_index::hashed_unique<
-                boost::multi_index::composite_key<
-                    ConnectingClient4,
-                    boost::multi_index::member<ConnectingClient4, std::vector<uint8_t>,
-                                               &ConnectingClient4::hwaddr_>,
-                    boost::multi_index::member<ConnectingClient4, std::vector<uint8_t>,
-                                               &ConnectingClient4::clientid_>
-                >
-            >,
+            ClientIdent4<ConnectingClient4>,
             // Second index allows for counting all clients which are
             // considered unacked.
             boost::multi_index::ordered_non_unique<
@@ -719,6 +945,31 @@ protected:
     /// the partner server while the servers are in communications
     /// interrupted state.
     ConnectingClients4 connecting_clients_;
+
+    /// @brief Structure holding information about the client who has a
+    /// rejected lease update.
+    struct RejectedClient4 {
+        std::vector<uint8_t> hwaddr_;
+        std::vector<uint8_t> clientid_;
+        int64_t expire_;
+    };
+
+    /// @brief Multi index container holding information about the clients
+    /// who have rejected leases.
+    typedef boost::multi_index_container<
+        RejectedClient4,
+        boost::multi_index::indexed_by<
+            ClientIdent4<RejectedClient4>,
+            boost::multi_index::ordered_non_unique<
+                boost::multi_index::member<RejectedClient4, int64_t,
+                                           &RejectedClient4::expire_>
+            >
+        >
+    > RejectedClients4;
+
+    /// @brief Holds information about the clients for whom lease updates
+    /// have been rejected by the partner.
+    RejectedClients4 rejected_clients_;
 };
 
 /// @brief Pointer to the @c CommunicationState4 object.
@@ -776,6 +1027,46 @@ public:
     /// @return Number of unacked clients.
     virtual size_t getUnackedClientsCount() const;
 
+protected:
+
+    /// @brief Returns the number of lease updates rejected by the partner.
+    ///
+    /// Each rejected lease update is counted only once if it failed
+    /// multiple times. Before returning the counter, it discards expired
+    /// rejected lease updates.
+    ///
+    /// @return Current rejected lease update number count.
+    virtual size_t getRejectedLeaseUpdatesCountInternal();
+
+    /// @brief Marks that the lease update failed due to a conflict for the
+    /// specified DHCP message.
+    ///
+    /// If the conflict has been already reported for the given client, the
+    /// rejected lease count remains unchanged.
+    ///
+    /// @param message DHCP message for which a lease update failed due to
+    ///  a conflict.
+    /// @param lifetime a time in seconds after which the rejected lease
+    /// update entry should be discarded.
+    /// @return true if the update was rejected for the first time, false
+    /// otherwise.
+    virtual bool reportRejectedLeaseUpdateInternal(const dhcp::PktPtr& message,
+                                                   const uint32_t lifetime = 86400);
+
+    /// @brief Marks the lease update successful.
+    ///
+    /// If the lease update was previously marked "in conflict", it is
+    /// now cleared, effectively lowering the number of conflicted leases.
+    ///
+    /// @param message DHCP message for which the lease update was
+    /// successful.
+    /// @return true when the lease was marked "in conflict" and it is
+    /// now cleared.
+    virtual bool reportSuccessfulLeaseUpdateInternal(const dhcp::PktPtr& message);
+
+    /// @brief Clears rejected client leases.
+    virtual void clearRejectedLeaseUpdatesInternal();
+
 protected:
 
     /// @brief Checks if the DHCPv6 message appears to be unanswered.
@@ -806,6 +1097,16 @@ protected:
     /// See @c CommunicationState::analyzeMessage for details.
     virtual void clearConnectingClients();
 
+    /// @brief Hashed index used in the multi index containers to find
+    /// clients by DUID.
+    ///
+    /// @tparam ClientData Type of a structure holding client information.
+    template<typename ClientData>
+    using ClientIdent6 = boost::multi_index::hashed_unique<
+        boost::multi_index::member<ClientData, std::vector<uint8_t>,
+                                   &ClientData::duid_>
+        >;
+
     /// @brief Structure holding information about a client which
     /// sent a packet being analyzed.
     struct ConnectingClient6 {
@@ -819,10 +1120,7 @@ protected:
         ConnectingClient6,
         boost::multi_index::indexed_by<
             // First index is for accessing connecting clients by DUID.
-            boost::multi_index::hashed_unique<
-                boost::multi_index::member<ConnectingClient6, std::vector<uint8_t>,
-                                           &ConnectingClient6::duid_>
-            >,
+            ClientIdent6<ConnectingClient6>,
             // Second index allows for counting all clients which are
             // considered unacked.
             boost::multi_index::ordered_non_unique<
@@ -835,6 +1133,30 @@ protected:
     /// the partner server while the servers are in communications
     /// interrupted state.
     ConnectingClients6 connecting_clients_;
+
+    /// @brief Structure holding information about the client who has a
+    /// rejected lease update.
+    struct RejectedClient6 {
+        std::vector<uint8_t> duid_;
+        int64_t expire_;
+    };
+
+    /// @brief Multi index container holding information about the clients
+    /// who have rejected leases.
+    typedef boost::multi_index_container<
+        RejectedClient6,
+        boost::multi_index::indexed_by<
+            ClientIdent6<RejectedClient6>,
+            boost::multi_index::ordered_non_unique<
+                boost::multi_index::member<RejectedClient6, int64_t,
+                                           &RejectedClient6::expire_>
+            >
+        >
+    > RejectedClients6;
+
+    /// @brief Holds information about the clients for whom lease updates
+    /// have been rejected by the partner.
+    RejectedClients6 rejected_clients_;
 };
 
 /// @brief Pointer to the @c CommunicationState6 object.
