@@ -1,4 +1,4 @@
-// Copyright (C) 2020-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2020-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -243,7 +243,7 @@ CloseHATest::runPartners(bool const backup /* = true */) {
             if (accept_partner2 > nfd) {
                 nfd = accept_partner2;
             }
-            for (auto reader : readers) {
+            for (auto const& reader : readers) {
                 if (!reader.second) {
                     continue;
                 }
@@ -285,7 +285,7 @@ CloseHATest::runPartners(bool const backup /* = true */) {
                     readers[fd] = true;
                 }
             }
-            for (auto reader : readers) {
+            for (auto const& reader : readers) {
                 if (!reader.second) {
                     continue;
                 }
@@ -314,7 +314,7 @@ CloseHATest::runPartners(bool const backup /* = true */) {
     if (accept_partner2 >= 0) {
         close(accept_partner2);
     }
-    for (auto reader : readers) {
+    for (auto const& reader : readers) {
         if (!reader.second) {
             continue;
         }
@@ -373,14 +373,13 @@ TEST_F(CloseHATest, close4) {
 
     // Check that the disabled state is reset on dhcp4_srv_configured.
     ASSERT_TRUE(network_state->isServiceEnabled());
-    network_state->disableService(NetworkState::Origin::HA_COMMAND);
+    network_state->disableService(NetworkState::HA_LOCAL_COMMAND);
     ASSERT_FALSE(network_state->isServiceEnabled());
 
     // Start HA service.
     EXPECT_TRUE(HooksManager::calloutsPresent(testHooks.hook_index_dhcp4_srv_configured_));
     {
         CalloutHandlePtr handle = HooksManager::createCalloutHandle();
-        handle->setArgument("io_context", io_service);
         handle->setArgument("network_state", network_state);
         HooksManager::callCallouts(testHooks.hook_index_dhcp4_srv_configured_,
                                    *handle);
@@ -508,14 +507,13 @@ TEST_F(CloseHATest, close4Backup) {
     // It is first reset by the constructor and then adjusted by running the
     // state model.
     ASSERT_TRUE(network_state->isServiceEnabled());
-    network_state->disableService(NetworkState::Origin::HA_COMMAND);
+    network_state->disableService(NetworkState::HA_LOCAL_COMMAND);
     ASSERT_FALSE(network_state->isServiceEnabled());
 
     // Start HA service.
     EXPECT_TRUE(HooksManager::calloutsPresent(testHooks.hook_index_dhcp4_srv_configured_));
     {
         CalloutHandlePtr handle = HooksManager::createCalloutHandle();
-        handle->setArgument("io_context", io_service);
         handle->setArgument("network_state", network_state);
         HooksManager::callCallouts(testHooks.hook_index_dhcp4_srv_configured_,
                                    *handle);
@@ -612,14 +610,13 @@ TEST_F(CloseHATest, close6) {
 
     // Check that the disabled state is reset on dhcp6_srv_configured.
     ASSERT_TRUE(network_state->isServiceEnabled());
-    network_state->disableService(NetworkState::Origin::HA_COMMAND);
+    network_state->disableService(NetworkState::HA_LOCAL_COMMAND);
     ASSERT_FALSE(network_state->isServiceEnabled());
 
     // Start HA service.
     EXPECT_TRUE(HooksManager::calloutsPresent(testHooks.hook_index_dhcp6_srv_configured_));
     {
         CalloutHandlePtr handle = HooksManager::createCalloutHandle();
-        handle->setArgument("io_context", io_service);
         handle->setArgument("network_state", network_state);
         HooksManager::callCallouts(testHooks.hook_index_dhcp6_srv_configured_,
                                    *handle);
@@ -747,14 +744,13 @@ TEST_F(CloseHATest, close6Backup) {
     // It is first reset by the constructor and then adjusted by running the
     // state model.
     ASSERT_TRUE(network_state->isServiceEnabled());
-    network_state->disableService(NetworkState::Origin::HA_COMMAND);
+    network_state->disableService(NetworkState::HA_LOCAL_COMMAND);
     ASSERT_FALSE(network_state->isServiceEnabled());
 
     // Start HA service.
     EXPECT_TRUE(HooksManager::calloutsPresent(testHooks.hook_index_dhcp6_srv_configured_));
     {
         CalloutHandlePtr handle = HooksManager::createCalloutHandle();
-        handle->setArgument("io_context", io_service);
         handle->setArgument("network_state", network_state);
         HooksManager::callCallouts(testHooks.hook_index_dhcp6_srv_configured_,
                                    *handle);
