@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,6 +8,7 @@
 
 #include <gtest/gtest.h>
 
+#include <testutils/sandbox.h>
 #include <asiolink/io_service.h>
 #include <config/base_command_mgr.h>
 #include <config/command_mgr.h>
@@ -28,6 +29,7 @@ using namespace std;
 // Test class for Command Manager
 class CommandMgrTest : public ::testing::Test {
 public:
+    isc::test::Sandbox sandbox;
 
     /// Default constructor
     CommandMgrTest()
@@ -40,7 +42,7 @@ public:
         handler_called_ = false;
         callout_name_ = "";
         callout_argument_names_.clear();
-        std::string processed_log_ = "";
+        processed_log_ = "";
 
         CommandMgr::instance().deregisterAll();
         CommandMgr::instance().closeCommandSocket();
@@ -58,13 +60,12 @@ public:
     /// @brief Returns socket path (using either hardcoded path or env variable)
     /// @return path to the unix socket
     std::string getSocketPath() {
-
         std::string socket_path;
         const char* env = getenv("KEA_SOCKET_TEST_DIR");
         if (env) {
             socket_path = std::string(env) + "/test-socket";
         } else {
-            socket_path = std::string(TEST_DATA_BUILDDIR) + "/test-socket";
+            socket_path = sandbox.join("test-socket");
         }
         return (socket_path);
     }
@@ -79,8 +80,8 @@ public:
         // Iterate over existing hook points and for each of them remove
         // callouts registered.
         std::vector<std::string> hooks = ServerHooks::getServerHooksPtr()->getHookNames();
-        for (auto h = hooks.cbegin(); h != hooks.cend(); ++h) {
-                HooksManager::preCalloutsLibraryHandle().deregisterAllCallouts(*h);
+        for (auto const& h : hooks) {
+            HooksManager::preCalloutsLibraryHandle().deregisterAllCallouts(h);
         }
     }
 
@@ -417,9 +418,8 @@ TEST_F(CommandMgrTest, delegateListCommands) {
     const std::vector<ElementPtr>& commands_list = answer_arg->listValue();
     ASSERT_EQ(3, commands_list.size());
     std::vector<std::string> command_names_list;
-    for (auto cmd = commands_list.cbegin(); cmd != commands_list.cend();
-         ++cmd) {
-        command_names_list.push_back((*cmd)->stringValue());
+    for (auto const& cmd : commands_list) {
+        command_names_list.push_back(cmd->stringValue());
     }
     std::sort(command_names_list.begin(), command_names_list.end());
     EXPECT_EQ("list-commands", command_names_list[0]);
@@ -542,3 +542,28 @@ TEST_F(CommandMgrTest, commandProcessedHookReplaceResponse) {
              "{ \"result\": 2, \"text\": \"'change-response' command not supported.\" }",
               processed_log_);
 }
+
+// Verifies that a socket cannot be concurrently opened more than once.
+TEST_F(CommandMgrTest, exclusiveOpen) {
+    // Pass in valid parameters.
+    ElementPtr socket_info = Element::createMap();
+    socket_info->set("socket-type", Element::create("unix"));
+    socket_info->set("socket-name", Element::create(getSocketPath()));
+
+    EXPECT_NO_THROW(CommandMgr::instance().openCommandSocket(socket_info));
+    EXPECT_GE(CommandMgr::instance().getControlSocketFD(), 0);
+
+    // Should not be able to open it twice.
+    EXPECT_THROW(CommandMgr::instance().openCommandSocket(socket_info),
+                 isc::config::SocketError);
+
+    // Now let's close it.
+    EXPECT_NO_THROW(CommandMgr::instance().closeCommandSocket());
+
+    // Should be able to re-open it now.
+    EXPECT_NO_THROW(CommandMgr::instance().openCommandSocket(socket_info));
+    EXPECT_GE(CommandMgr::instance().getControlSocketFD(), 0);
+
+    // Now let's close it.
+    EXPECT_NO_THROW(CommandMgr::instance().closeCommandSocket());
+}
