@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -70,8 +70,6 @@ CryptoLink::initialize(CryptoLink& c) {
             isc_throw(InitializationError, "Botan error: " << ex.what());
         }
     }
-    // A not yet fixed bug makes RNG to be destroyed after memory pool...
-    atexit([]{ getCryptoLink().getRNG().reset(); });
 }
 
 std::string
