@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,6 +7,7 @@
 #include <config.h>
 
 #include <asiolink/io_address.h>
+#include <dhcp/testutils/pkt_captures.h>
 #include <dhcp/dhcp6.h>
 #include <dhcp/option.h>
 #include <dhcp/option_custom.h>
@@ -21,13 +22,13 @@
 #include <dhcp/pkt6.h>
 #include <dhcp/hwaddr.h>
 #include <dhcp/docsis3_option_defs.h>
-#include <dhcp/tests/pkt_captures.h>
 #include <testutils/gtest_utils.h>
 #include <util/range_utilities.h>
 #include <boost/date_time/posix_time/posix_time.hpp>
+#include <boost/range/adaptor/reversed.hpp>
 #include <boost/scoped_ptr.hpp>
 #include <boost/pointer_cast.hpp>
-#include <util/encode/hex.h>
+#include <util/encode/encode.h>
 #include <gtest/gtest.h>
 
 #include <algorithm>
@@ -452,9 +453,8 @@ TEST_F(Pkt6Test, addGetDelOptions) {
 
     // Both options must be of type 2 and there must not be
     // any other type returned
-    for (OptionCollection::const_iterator x= options.begin();
-         x != options.end(); ++x) {
-        EXPECT_EQ(2, x->second->getType());
+    for (auto const& x : options) {
+        EXPECT_EQ(2, x.second->getType());
     }
 
     // Try to get a single option. Normally for singular options
@@ -552,9 +552,8 @@ TEST_F(Pkt6Test, getOptions) {
     // that copies of the options were used to replace original options
     // in the packet.
     OptionCollection options_modified = pkt.getNonCopiedOptions(1);
-    for (OptionCollection::const_iterator opt_it_modified = options_modified.begin();
-         opt_it_modified != options_modified.end(); ++opt_it_modified) {
-        opt_it = std::find(options.begin(), options.end(), *opt_it_modified);
+    for (auto const& opt_it_modified : options_modified) {
+        opt_it = std::find(options.begin(), options.end(), opt_it_modified);
         ASSERT_TRUE(opt_it != options.end());
     }
 
@@ -1029,7 +1028,7 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
         msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_CLIENT);
     EXPECT_EQ(3, opts0.size());
     vector<OptionPtr> lopts0;
-    for (auto it : opts0) {
+    for (auto const& it : opts0) {
         lopts0.push_back(it.second);
     }
     ASSERT_EQ(3, lopts0.size());
@@ -1052,7 +1051,7 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_SERVER);
     EXPECT_EQ(3, opts.size());
     vector<OptionPtr> lopts;
-    for (auto it : opts) {
+    for (auto const& it : opts) {
         lopts.push_back(it.second);
     }
     ASSERT_EQ(3, lopts.size());
@@ -1064,9 +1063,10 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
 
     // Check reverse order.
     vector<OptionPtr> ropts;
-    for (auto it = opts.rbegin(); it != opts.rend(); ++it) {
-        ropts.push_back(it->second);
+    for (auto const& it : boost::adaptors::reverse(opts)) {
+         ropts.push_back(it.second);
     }
+
     EXPECT_TRUE(lopts0 == ropts);
 
     // We just want option from the first relay (closest to the client)
@@ -1113,7 +1113,7 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     // Check collections.
     opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_CLIENT);
     lopts0.clear();
-    for (auto it : opts) {
+    for (auto const& it : opts) {
         lopts0.push_back(it.second);
     }
     ASSERT_EQ(3, lopts0.size());
@@ -1123,7 +1123,7 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     EXPECT_TRUE(lopts0[2] == relay1_opt1);
     opts = msg->getAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_CLIENT);
     lopts.clear();
-    for (auto it : opts) {
+    for (auto const& it : opts) {
         lopts.push_back(it.second);
     }
     ASSERT_EQ(3, lopts.size());
@@ -1150,7 +1150,7 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     // Check collections.
     opts = msg->getNonCopiedAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_SERVER);
     lopts0.clear();
-    for (auto it : opts) {
+    for (auto const& it : opts) {
         lopts0.push_back(it.second);
     }
     ASSERT_EQ(3, lopts0.size());
@@ -1160,7 +1160,7 @@ TEST_F(Pkt6Test, getAnyRelayOption) {
     EXPECT_TRUE(lopts0[2] == relay3_opt1);
     opts = msg->getAllRelayOptions(200, Pkt6::RELAY_SEARCH_FROM_SERVER);
     lopts.clear();
-    for (auto it : opts) {
+    for (auto const& it : opts) {
         lopts.push_back(it.second);
     }
     ASSERT_EQ(3, lopts.size());
@@ -1287,12 +1287,13 @@ TEST_F(Pkt6Test, toText) {
     ASSERT_EQ(2, msg->relay_info_.size());
 
     string expected =
-        "localAddr=[ff05::1:3]:547 remoteAddr=[fe80::1234]:547\n"
-        "msgtype=1(SOLICIT), transid=0x6b4fe2\n"
-        "type=00001, len=00014: 00:01:00:01:18:b0:33:41:00:00:21:5c:18:a9\n"
-        "type=00003(IA_NA), len=00012: iaid=1, t1=4294967295, t2=4294967295\n"
-        "type=00006, len=00006: 23(uint16) 242(uint16) 243(uint16)\n"
-        "type=00008, len=00002: 0 (uint16)\n"
+        "local_address=[ff05::1:3]:547, remote_address=[fe80::1234]:547,\n"
+        "msg_type=SOLICIT (1), trans_id=0x6b4fe2,\n"
+        "options:\n"
+        "  type=00001, len=00014: 00:01:00:01:18:b0:33:41:00:00:21:5c:18:a9\n"
+        "  type=00003(IA_NA), len=00012: iaid=1, t1=4294967295, t2=4294967295\n"
+        "  type=00006, len=00006: 23(uint16) 242(uint16) 243(uint16)\n"
+        "  type=00008, len=00002: 0 (uint16)\n"
         "2 relay(s):\n"
         "relay[0]: msg-type=12(RELAY_FORWARD), hop-count=1,\n"
         "link-address=2001:888:db8:1::, peer-address=fe80::200:21ff:fe5c:18a9, 2 option(s)\n"
@@ -1963,7 +1964,7 @@ TEST_F(Pkt6Test, makeLabel) {
                                                  HTYPE_ETHER)));
 
     // Specify DUID and no HW Address.
-    EXPECT_EQ("duid=[01:02:02:02:02:03:03:03:03:03:03], tid=0x123",
+    EXPECT_EQ("duid=[01:02:02:02:02:03:03:03:03:03:03], [no hwaddr info], tid=0x123",
               Pkt6::makeLabel(duid, 0x123, HWAddrPtr()));
 
     // Specify HW Address and no DUID.
@@ -1976,7 +1977,7 @@ TEST_F(Pkt6Test, makeLabel) {
               Pkt6::makeLabel(duid, 0x123, hwaddr));
 
     // Specify neither DUID nor HW Address.
-    EXPECT_EQ("duid=[no info], tid=0x0",
+    EXPECT_EQ("duid=[no info], [no hwaddr info], tid=0x0",
               Pkt6::makeLabel(DuidPtr(), 0x0, HWAddrPtr()));
 }
 
@@ -1988,7 +1989,7 @@ TEST_F(Pkt6Test, makeLabelWithoutTransactionId) {
                                                  HTYPE_ETHER)));
 
     // Specify DUID and no HW Address.
-    EXPECT_EQ("duid=[01:02:02:02:02:03:03:03:03:03:03]",
+    EXPECT_EQ("duid=[01:02:02:02:02:03:03:03:03:03:03], [no hwaddr info]",
               Pkt6::makeLabel(duid, HWAddrPtr()));
 
     // Specify HW Address and no DUID.
@@ -2001,7 +2002,7 @@ TEST_F(Pkt6Test, makeLabelWithoutTransactionId) {
               Pkt6::makeLabel(duid, hwaddr));
 
     // Specify neither DUID nor HW Address.
-    EXPECT_EQ("duid=[no info]", Pkt6::makeLabel(DuidPtr(), HWAddrPtr()));
+    EXPECT_EQ("duid=[no info], [no hwaddr info]", Pkt6::makeLabel(DuidPtr(), HWAddrPtr()));
 }
 
 // This test verifies that it is possible to obtain the packet
@@ -2009,7 +2010,7 @@ TEST_F(Pkt6Test, makeLabelWithoutTransactionId) {
 TEST_F(Pkt6Test, getLabel) {
     // Create a packet.
     Pkt6Ptr pkt(new Pkt6(DHCPV6_SOLICIT, 0x2312));
-    EXPECT_EQ("duid=[no info], tid=0x2312",
+    EXPECT_EQ("duid=[no info], [no hwaddr info], tid=0x2312",
               pkt->getLabel());
 
     DuidPtr duid(new DUID(DUID::fromText("0102020202030303030303")));
@@ -2022,7 +2023,7 @@ TEST_F(Pkt6Test, getLabel) {
     Pkt6Ptr pkt_clone = packAndClone(pkt);
     ASSERT_NO_THROW(pkt_clone->unpack());
 
-    EXPECT_EQ("duid=[01:02:02:02:02:03:03:03:03:03:03], tid=0x2312",
+    EXPECT_EQ("duid=[01:02:02:02:02:03:03:03:03:03:03], [no hwaddr info], tid=0x2312",
               pkt_clone->getLabel());
 
 }
@@ -2035,7 +2036,7 @@ TEST_F(Pkt6Test, getLabelEmptyClientId) {
 
     // Add empty client identifier option.
     pkt.addOption(OptionPtr(new Option(Option::V6, D6O_CLIENTID)));
-    EXPECT_EQ("duid=[no info], tid=0x2312", pkt.getLabel());
+    EXPECT_EQ("duid=[no info], [no hwaddr info], tid=0x2312", pkt.getLabel());
 }
 
 // Verifies that when the VIVSO, 17, has length that is too
@@ -2370,4 +2371,70 @@ TEST_F(Pkt6Test, relayDataOption) {
     EXPECT_EQ(orig_data, clone_data);
 }
 
+// Exercises packet event stack and helper functions.
+TEST_F(Pkt6Test, PktEvents) {
+    // Get current time.
+    auto start_time = PktEvent::now();
+
+    // Verify that a set time is not equal to an EMPTY_TIME.
+    ASSERT_NE(start_time, PktEvent::EMPTY_TIME());
+
+    // Create a test packet.
+    scoped_ptr<Pkt6> pkt(new Pkt6(DHCPV6_SOLICIT, 0x020304));
+
+    // Upon creation, the events table should be empty.
+    ASSERT_TRUE(pkt->getPktEvents().empty());
+
+    // An non-existent event should return an empty time.
+    auto event_time = pkt->getPktEventTime(PktEvent::BUFFER_READ);
+    ASSERT_EQ(event_time, PktEvent::EMPTY_TIME());
+
+    // Sleep for 200 microseconds to put some distance between now and start_time.
+    usleep(200);
+
+    // Should be able to add an event, defaulting the event time to current time.
+    pkt->addPktEvent(PktEvent::BUFFER_READ);
+    event_time = pkt->getPktEventTime(PktEvent::BUFFER_READ);
+    ASSERT_GT(event_time, start_time);
+
+    // Should be able to overwrite an existing event's time.
+    pkt->setPktEvent(PktEvent::BUFFER_READ, start_time);
+    event_time = pkt->getPktEventTime(PktEvent::BUFFER_READ);
+    ASSERT_EQ(event_time, start_time);
+
+    // Should be able to add an event with an explicit time.
+    pkt->addPktEvent(PktEvent::RESPONSE_SENT, start_time);
+    event_time = pkt->getPktEventTime(PktEvent::RESPONSE_SENT);
+    ASSERT_EQ(event_time, start_time);
+
+    // Should be able to fetch the list of events.
+    auto const& events = pkt->getPktEvents();
+    ASSERT_FALSE(events.empty());
+    auto event = events.begin();
+    ASSERT_EQ((*event).label_, PktEvent::BUFFER_READ);
+    ++event;
+    ASSERT_EQ((*event).label_, PktEvent::RESPONSE_SENT);
+
+    // Discard the event stack contents.
+    pkt->clearPktEvents();
+    ASSERT_TRUE(pkt->getPktEvents().empty());
+
+    // Verify dumpPktEvent terse output. Also serves to
+    // verify adding events using struct timeval.
+    struct timeval log_time = {1706802676, 100};
+    struct timeval log_time_plus = {1706802676, 250};
+    pkt->addPktEvent("first-event", log_time);
+    pkt->addPktEvent("second-event", log_time_plus);
+    std::string log = pkt->dumpPktEvents();
+    EXPECT_EQ(log, "2024-Feb-01 15:51:16.000100 : first-event, 2024-Feb-01 15:51:16.000250 : second-event");
+
+    // Verify dumpPktEvent verbose output.
+    log = pkt->dumpPktEvents(true);
+    EXPECT_EQ(log,
+              "Event log: \n"
+              "2024-Feb-01 15:51:16.000100 : first-event\n"
+              "2024-Feb-01 15:51:16.000250 : second-event elapsed: 00:00:00.000150\n"
+              "total elapsed: 00:00:00.000150");
+}
+
 }  // namespace
