@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -60,17 +60,17 @@ CSVLeaseFile4::append(const Lease4& lease) {
     }
 
     row.writeAt(getColumnIndex("valid_lifetime"), lease.valid_lft_);
-    row.writeAt(getColumnIndex("expire"), static_cast<uint64_t>(lease.cltt_ + lease.valid_lft_));
+    row.writeAt(getColumnIndex("expire"), static_cast<uint64_t>(lease.cltt_) + lease.valid_lft_);
     row.writeAt(getColumnIndex("subnet_id"), lease.subnet_id_);
     row.writeAt(getColumnIndex("fqdn_fwd"), lease.fqdn_fwd_);
     row.writeAt(getColumnIndex("fqdn_rev"), lease.fqdn_rev_);
-    row.writeAt(getColumnIndex("hostname"), lease.hostname_);
+    row.writeAtEscaped(getColumnIndex("hostname"), lease.hostname_);
     row.writeAt(getColumnIndex("state"), lease.state_);
     // User context is optional.
     if (lease.getContext()) {
-        row.writeAt(getColumnIndex("user_context"), lease.getContext()->str());
+        row.writeAtEscaped(getColumnIndex("user_context"), lease.getContext()->str());
     }
-
+    row.writeAt(getColumnIndex("pool_id"), lease.pool_id_);
     try {
         VersionedCSVFile::append(row);
     } catch (const std::exception&) {
@@ -140,13 +140,15 @@ CSVLeaseFile4::next(Lease4Ptr& lease) {
                                readFqdnFwd(row),
                                readFqdnRev(row),
                                readHostname(row)));
+
         lease->state_ = state;
 
         if (ctx) {
             lease->setContext(ctx);
         }
 
-    } catch (std::exception& ex) {
+        lease->pool_id_ = readPoolID(row);
+    } catch (const std::exception& ex) {
         // bump the read error count
         ++read_errs_;
 
@@ -176,6 +178,8 @@ CSVLeaseFile4::initColumns() {
     addColumn("hostname", "1.0");
     addColumn("state", "2.0", "0");
     addColumn("user_context", "2.1");
+    addColumn("pool_id", "3.0", "0");
+
     // Any file with less than hostname is invalid
     setMinimumValidColumns("hostname");
 }
@@ -225,6 +229,13 @@ CSVLeaseFile4::readSubnetID(const CSVRow& row) {
     return (subnet_id);
 }
 
+uint32_t
+CSVLeaseFile4::readPoolID(const CSVRow& row) {
+    uint32_t pool_id =
+        row.readAndConvertAt<uint32_t>(getColumnIndex("pool_id"));
+    return (pool_id);
+}
+
 bool
 CSVLeaseFile4::readFqdnFwd(const CSVRow& row) {
     bool fqdn_fwd = row.readAndConvertAt<bool>(getColumnIndex("fqdn_fwd"));
@@ -239,7 +250,7 @@ CSVLeaseFile4::readFqdnRev(const CSVRow& row) {
 
 std::string
 CSVLeaseFile4::readHostname(const CSVRow& row) {
-    std::string hostname = row.readAt(getColumnIndex("hostname"));
+    std::string hostname = row.readAtEscaped(getColumnIndex("hostname"));
     return (hostname);
 }
 
@@ -251,7 +262,7 @@ CSVLeaseFile4::readState(const util::CSVRow& row) {
 
 ConstElementPtr
 CSVLeaseFile4::readContext(const util::CSVRow& row) {
-    std::string user_context = row.readAt(getColumnIndex("user_context"));
+    std::string user_context = row.readAtEscaped(getColumnIndex("user_context"));
     if (user_context.empty()) {
         return (ConstElementPtr());
     }
