@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,17 +8,20 @@
 
 #include <dhcp_ddns/ncr_udp.h>
 #include <dhcpsrv/d2_client_cfg.h>
+#include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/dhcpsrv_log.h>
 
-#include <boost/algorithm/string/predicate.hpp>
-
 #include <string>
 
 using namespace std;
+using namespace isc::asiolink;
+using namespace isc::data;
+using namespace isc::util;
 
 namespace isc {
 namespace dhcp {
 
+/// These values need to match those used in D2ClientConfigParser::SimpleDefaults
 const char* D2ClientConfig::DFT_SERVER_IP = "127.0.0.1";
 const size_t D2ClientConfig::DFT_SERVER_PORT = 53001;
 const char* D2ClientConfig::DFT_V4_SENDER_IP = "0.0.0.0";
@@ -27,28 +30,28 @@ const size_t D2ClientConfig::DFT_SENDER_PORT = 0;
 const size_t D2ClientConfig::DFT_MAX_QUEUE_SIZE = 1024;
 const char* D2ClientConfig::DFT_NCR_PROTOCOL = "UDP";
 const char* D2ClientConfig::DFT_NCR_FORMAT = "JSON";
-const bool D2ClientConfig::DFT_ALWAYS_INCLUDE_FQDN = false;
 const bool D2ClientConfig::DFT_OVERRIDE_NO_UPDATE = false;
 const bool D2ClientConfig::DFT_OVERRIDE_CLIENT_UPDATE = false;
 const char* D2ClientConfig::DFT_REPLACE_CLIENT_NAME_MODE = "NEVER";
 const char* D2ClientConfig::DFT_GENERATED_PREFIX = "myhost";
-
+const char* D2ClientConfig::DFT_HOSTNAME_CHAR_SET = "";
+const char* D2ClientConfig::DFT_HOSTNAME_CHAR_REPLACEMENT = "";
 
 D2ClientConfig::ReplaceClientNameMode
 D2ClientConfig::stringToReplaceClientNameMode(const std::string& mode_str) {
-    if (boost::iequals(mode_str, "never")) {
+    if (mode_str == "never") {
         return (D2ClientConfig::RCM_NEVER);
     }
 
-    if (boost::iequals(mode_str, "always")) {
+    if (mode_str == "always") {
         return (D2ClientConfig::RCM_ALWAYS);
     }
 
-    if (boost::iequals(mode_str, "when-present")) {
+    if (mode_str == "when-present") {
         return (D2ClientConfig::RCM_WHEN_PRESENT);
     }
 
-    if (boost::iequals(mode_str, "when-not-present")) {
+    if (mode_str == "when-not-present") {
         return (D2ClientConfig::RCM_WHEN_NOT_PRESENT);
     }
 
@@ -85,13 +88,7 @@ D2ClientConfig::D2ClientConfig(const  bool enable_updates,
                                const dhcp_ddns::
                                      NameChangeProtocol& ncr_protocol,
                                const dhcp_ddns::
-                                     NameChangeFormat& ncr_format,
-                               const bool always_include_fqdn,
-                               const bool override_no_update,
-                               const bool override_client_update,
-                               const ReplaceClientNameMode replace_client_name_mode,
-                               const std::string& generated_prefix,
-                               const std::string& qualifying_suffix)
+                                     NameChangeFormat& ncr_format)
     : enable_updates_(enable_updates),
       server_ip_(server_ip),
       server_port_(server_port),
@@ -99,13 +96,7 @@ D2ClientConfig::D2ClientConfig(const  bool enable_updates,
       sender_port_(sender_port),
       max_queue_size_(max_queue_size),
       ncr_protocol_(ncr_protocol),
-      ncr_format_(ncr_format),
-      always_include_fqdn_(always_include_fqdn),
-      override_no_update_(override_no_update),
-      override_client_update_(override_client_update),
-      replace_client_name_mode_(replace_client_name_mode),
-      generated_prefix_(generated_prefix),
-      qualifying_suffix_(qualifying_suffix) {
+      ncr_format_(ncr_format) {
     validateContents();
 }
 
@@ -117,13 +108,7 @@ D2ClientConfig::D2ClientConfig()
       sender_port_(DFT_SENDER_PORT),
       max_queue_size_(DFT_MAX_QUEUE_SIZE),
       ncr_protocol_(dhcp_ddns::stringToNcrProtocol(DFT_NCR_PROTOCOL)),
-      ncr_format_(dhcp_ddns::stringToNcrFormat(DFT_NCR_FORMAT)),
-      always_include_fqdn_(DFT_ALWAYS_INCLUDE_FQDN),
-      override_no_update_(DFT_OVERRIDE_NO_UPDATE),
-      override_client_update_(DFT_OVERRIDE_CLIENT_UPDATE),
-      replace_client_name_mode_(stringToReplaceClientNameMode(DFT_REPLACE_CLIENT_NAME_MODE)),
-      generated_prefix_(DFT_GENERATED_PREFIX),
-      qualifying_suffix_("") {
+      ncr_format_(dhcp_ddns::stringToNcrFormat(DFT_NCR_FORMAT)) {
     validateContents();
 }
 
@@ -175,13 +160,7 @@ D2ClientConfig::operator == (const D2ClientConfig& other) const {
             (sender_port_ == other.sender_port_) &&
             (max_queue_size_ == other.max_queue_size_) &&
             (ncr_protocol_ == other.ncr_protocol_) &&
-            (ncr_format_ == other.ncr_format_) &&
-            (always_include_fqdn_ == other.always_include_fqdn_) &&
-            (override_no_update_ == other.override_no_update_) &&
-            (override_client_update_ == other.override_client_update_) &&
-            (replace_client_name_mode_ == other.replace_client_name_mode_) &&
-            (generated_prefix_ == other.generated_prefix_) &&
-            (qualifying_suffix_ == other.qualifying_suffix_));
+            (ncr_format_ == other.ncr_format_));
 }
 
 bool
@@ -195,28 +174,43 @@ D2ClientConfig::toText() const {
 
     stream << "enable_updates: " << (enable_updates_ ? "yes" : "no");
     if (enable_updates_) {
-        stream << ", server_ip: " << server_ip_.toText()
-               << ", server_port: " << server_port_
-               << ", sender_ip: " << sender_ip_.toText()
-               << ", sender_port: " << sender_port_
-               << ", max_queue_size: " << max_queue_size_
-               << ", ncr_protocol: " << ncrProtocolToString(ncr_protocol_)
-               << ", ncr_format: " << ncrFormatToString(ncr_format_)
-               << ", always_include_fqdn: " << (always_include_fqdn_ ?
-                                                "yes" : "no")
-               << ", override_no_update: " << (override_no_update_ ?
-                                               "yes" : "no")
-               << ", override_client_update: " << (override_client_update_ ?
-                                                   "yes" : "no")
-               << ", replace_client_name: "
-               << replaceClientNameModeToString(replace_client_name_mode_)
-               << ", generated_prefix: [" << generated_prefix_ << "]"
-               << ", qualifying_suffix: [" << qualifying_suffix_ << "]";
+        stream << ", server-ip: " << server_ip_.toText()
+               << ", server-port: " << server_port_
+               << ", sender-ip: " << sender_ip_.toText()
+               << ", sender-port: " << sender_port_
+               << ", max-queue-size: " << max_queue_size_
+               << ", ncr-protocol: " << ncrProtocolToString(ncr_protocol_)
+               << ", ncr-format: " << ncrFormatToString(ncr_format_);
     }
 
+
     return (stream.str());
 }
 
+ElementPtr
+D2ClientConfig::toElement() const {
+    ElementPtr result = Element::createMap();
+    // Set user context
+    contextToElement(result);
+    // Set enable-updates
+    result->set("enable-updates", Element::create(enable_updates_));
+    // Set server-ip
+    result->set("server-ip", Element::create(server_ip_.toText()));
+    // Set server-port
+    result->set("server-port", Element::create(static_cast<long long>(server_port_)));
+    // Set sender-ip
+    result->set("sender-ip", Element::create(sender_ip_.toText()));
+    // Set sender-port
+    result->set("sender-port", Element::create(static_cast<long long>(sender_port_)));
+    // Set max-queue-size
+    result->set("max-queue-size", Element::create(static_cast<long long>(max_queue_size_)));
+    // Set ncr-protocol
+    result->set("ncr-protocol", Element::create(dhcp_ddns::ncrProtocolToString(ncr_protocol_)));
+    // Set ncr-format
+    result->set("ncr-format", Element::create(dhcp_ddns::ncrFormatToString(ncr_format_)));
+    return (result);
+}
+
 std::ostream&
 operator<<(std::ostream& os, const D2ClientConfig& config) {
     os << config.toText();
@@ -224,5 +218,4 @@ operator<<(std::ostream& os, const D2ClientConfig& config) {
 }
 
 };  // namespace dhcp
-
 };  // namespace isc
