@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -227,12 +227,18 @@ public:
     ///
     /// Starts test timer which detects timeouts.
     HttpListenerTest()
-        : io_service_(), factory_(new TestHttpResponseCreatorFactory()),
+        : io_service_(new IOService()), factory_(new TestHttpResponseCreatorFactory()),
           test_timer_(io_service_), run_io_service_timer_(io_service_) {
         test_timer_.setup(std::bind(&HttpListenerTest::timeoutHandler, this, true),
                           TEST_TIMEOUT, IntervalTimer::ONE_SHOT);
     }
 
+    /// @brief Destructor.
+    virtual ~HttpListenerTest() {
+        test_timer_.cancel();
+        io_service_->stopAndPoll();
+    }
+
     /// @brief Callback function invoke upon test timeout.
     ///
     /// It stops the IO service and reports test timeout.
@@ -242,7 +248,7 @@ public:
         if (fail_on_timeout) {
             ADD_FAILURE() << "Timeout occurred while running the test!";
         }
-        io_service_.stop();
+        io_service_->stop();
     }
 
     /// @brief Runs IO service with optional timeout.
@@ -250,20 +256,20 @@ public:
     /// @param timeout Optional value specifying for how long the io service
     /// should be ran (ms).
     void runIOService(long timeout = 0) {
-        io_service_.get_io_service().reset();
+        io_service_->stop();
+        io_service_->restart();
 
         if (timeout > 0) {
             run_io_service_timer_.setup(std::bind(&HttpListenerTest::timeoutHandler,
                                                   this, false),
                                         timeout, IntervalTimer::ONE_SHOT);
         }
-        io_service_.run();
-        io_service_.get_io_service().reset();
-        io_service_.poll();
+        io_service_->run();
+        io_service_->stopAndPoll(false);
     }
 
     /// @brief IO service used in the tests.
-    IOService io_service_;
+    IOServicePtr io_service_;
 
     /// @brief Pointer to the response creator factory.
     HttpResponseCreatorFactoryPtr factory_;
@@ -315,7 +321,7 @@ public:
         listener_->stop();
         listener2_->stop();
         listener3_->stop();
-        io_service_.poll();
+        io_service_->poll();
         MultiThreadingMgr::instance().setMode(false);
         HttpRequest::recordSubject_ = false;
         HttpRequest::recordIssuer_ = false;
@@ -359,7 +365,7 @@ public:
         ASSERT_NO_THROW(listener_->start());
 
         // Create a client and specify the URL on which the server can be reached.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
         Url url("http://127.0.0.1:18123");
 
         // Initiate request to the server.
@@ -372,7 +378,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
             if (++resp_num > 1) {
-                io_service_.stop();
+                io_service_->stop();
             }
             if (ec) {
                 ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
@@ -388,7 +394,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
             if (++resp_num > 1) {
-                io_service_.stop();
+                io_service_->stop();
             }
             if (ec) {
                 ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
@@ -421,7 +427,7 @@ public:
         ASSERT_NO_THROW(listener2_->start());
 
         // Create the client. It will be communicating with the two servers.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URLs on which the servers are available.
         Url url1("http://127.0.0.1:18123");
@@ -437,7 +443,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
             if (++resp_num > 1) {
-                io_service_.stop();
+                io_service_->stop();
             }
             if (ec) {
                 ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
@@ -453,7 +459,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
             if (++resp_num > 1) {
-                io_service_.stop();
+                io_service_->stop();
             }
             if (ec) {
                 ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
@@ -482,7 +488,7 @@ public:
         ASSERT_NO_THROW(listener_->start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Create a second client context.
         TlsContextPtr client_context2;
@@ -501,7 +507,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
             if (++resp_num > 1) {
-                io_service_.stop();
+                io_service_->stop();
             }
             if (ec) {
                 ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
@@ -517,7 +523,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
             if (++resp_num > 1) {
-                io_service_.stop();
+                io_service_->stop();
             }
             if (ec) {
                 ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
@@ -550,7 +556,7 @@ public:
         ASSERT_NO_THROW(listener3_->start());
 
         // Create the client that will communicate with this server.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of this server.
         Url url("http://127.0.0.1:18125");
@@ -562,7 +568,7 @@ public:
                                                 request1, response1,
             [this](const boost::system::error_code& ec, const HttpResponsePtr&,
                    const std::string&) {
-            io_service_.stop();
+            io_service_->stop();
             if (ec) {
                 ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
             }
@@ -587,7 +593,7 @@ public:
                                                 request2, response2,
             [this](const boost::system::error_code& ec, const HttpResponsePtr&,
                    const std::string&) {
-            io_service_.stop();
+            io_service_->stop();
             if (ec) {
                 ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
             }
@@ -609,7 +615,7 @@ public:
     /// server is unreachable.
     void testUnreachable () {
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server. This server is down.
         Url url("http://127.0.0.1:18123");
@@ -622,7 +628,7 @@ public:
             [this](const boost::system::error_code& ec,
                    const HttpResponsePtr&,
                    const std::string&) {
-            io_service_.stop();
+            io_service_->stop();
             // The server should have returned an IO error.
             if (!ec) {
                 ADD_FAILURE() << "asyncSendRequest didn't fail";
@@ -640,7 +646,7 @@ public:
         ASSERT_NO_THROW(listener_->start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server.
         Url url("http://127.0.0.1:18123");
@@ -657,7 +663,7 @@ public:
             [this](const boost::system::error_code& ec,
                    const HttpResponsePtr& response,
                    const std::string& parsing_error) {
-            io_service_.stop();
+            io_service_->stop();
             // There should be no IO error (answer from the server is received).
             if (ec) {
                 ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
@@ -679,7 +685,7 @@ public:
         ASSERT_NO_THROW(listener_->start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server.
         Url url("http://127.0.0.1:18123");
@@ -700,7 +706,7 @@ public:
                             const HttpResponsePtr& response,
                             const std::string&) {
                 if (++cb_num > 1) {
-                    io_service_.stop();
+                    io_service_->stop();
                 }
                 // In this particular case we know exactly the type of the
                 // IO error returned, because the client explicitly sets this
@@ -728,7 +734,7 @@ public:
                             const HttpResponsePtr&,
                             const std::string&) {
                 if (++cb_num > 1) {
-                    io_service_.stop();
+                    io_service_->stop();
                 }
             }));
 
@@ -744,7 +750,7 @@ public:
         ASSERT_NO_THROW(listener_->start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server.
         Url url("http://127.0.0.1:18123");
@@ -759,7 +765,7 @@ public:
                             const HttpResponsePtr& response,
                             const std::string&) {
                 if (++cb_num > 1) {
-                    io_service_.stop();
+                    io_service_->stop();
                 }
                 // In this particular case we know exactly the type of the
                 // IO error returned, because the client explicitly sets this
@@ -786,7 +792,7 @@ public:
                             const HttpResponsePtr&,
                             const std::string&) {
                 if (++cb_num > 1) {
-                    io_service_.stop();
+                    io_service_->stop();
                 }
             }));
 
@@ -818,7 +824,7 @@ public:
         ASSERT_NO_THROW(listener_->start());
 
         // Create the client.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
 
         // Specify the URL of the server.
         Url url("http://127.0.0.1:18123");
@@ -880,7 +886,7 @@ public:
                          [this](const boost::system::error_code& ec,
                                 const HttpResponsePtr&,
                                 const std::string&) {
-            io_service_.stop();
+            io_service_->stop();
 
             // Everything should be ok.
             if (ec) {
@@ -903,7 +909,7 @@ public:
         ASSERT_NO_THROW(listener_->start());
 
         // Create a client and specify the URL on which the server can be reached.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
         Url url("http://127.0.0.1:18123");
 
         // Initiate request to the server.
@@ -918,7 +924,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
                 if (++resp_num > 1) {
-                    io_service_.stop();
+                    io_service_->stop();
                 }
 
                 if (ec) {
@@ -940,7 +946,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
                 if (++resp_num > 1) {
-                    io_service_.stop();
+                    io_service_->stop();
                 }
                 if (ec) {
                     ADD_FAILURE() << "asyncSendRequest failed: " << ec.message();
@@ -1002,7 +1008,7 @@ public:
         ASSERT_NO_THROW(listener_->start());
 
         // Create a client and specify the URL on which the server can be reached.
-        HttpClient client(io_service_);
+        HttpClient client(io_service_, false);
         Url url("http://127.0.0.1:18123");
 
         // Initiate request to the server.
@@ -1017,7 +1023,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
                 if (++resp_num == 1) {
-                    io_service_.stop();
+                    io_service_->stop();
                 }
 
                 // We should have 1 connect.
@@ -1084,7 +1090,7 @@ public:
                               const HttpResponsePtr&,
                               const std::string&) {
                 if (++resp_num == 1) {
-                    io_service_.stop();
+                    io_service_->stop();
                 }
 
                 // We should have 1 connect.
