@@ -1,10 +1,12 @@
-// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+
+#include <asiolink/io_service_mgr.h>
 #include <cc/command_interpreter.h>
 #include <process/cfgrpt/config_report.h>
 #include <cryptolink/crypto_hash.h>
@@ -13,7 +15,7 @@
 #include <hooks/hooks_manager.h>
 #include <log/logger.h>
 #include <log/logger_support.h>
-#include <util/encode/hex.h>
+#include <util/encode/encode.h>
 #include <process/daemon.h>
 #include <process/d_log.h>
 #include <process/d_controller.h>
@@ -50,8 +52,7 @@ DControllerBase::setController(const DControllerBasePtr& controller) {
     if (controller_) {
         // This shouldn't happen, but let's make sure it can't be done.
         // It represents a programmatic error.
-        isc_throw (DControllerBaseError,
-                "Multiple controller instances attempted.");
+        isc_throw(DControllerBaseError, "Multiple controller instances attempted.");
     }
 
     controller_ = controller;
@@ -104,7 +105,7 @@ DControllerBase::launch(int argc, char* argv[], const bool test_mode) {
     } catch (const std::exception& ex) {
         LOG_FATAL(dctl_logger, DCTL_CONFIG_FILE_LOAD_FAIL)
             .arg(app_name_).arg(ex.what());
-        isc_throw (LaunchError, "Launch Failed: " << ex.what());
+        isc_throw(LaunchError, "Launch Failed: " << ex.what());
     }
 
     try {
@@ -112,11 +113,11 @@ DControllerBase::launch(int argc, char* argv[], const bool test_mode) {
     } catch (const DaemonPIDExists& ex) {
         LOG_FATAL(dctl_logger, DCTL_ALREADY_RUNNING)
                   .arg(bin_name_).arg(ex.what());
-        isc_throw (LaunchError, "Launch Failed: " << ex.what());
+        isc_throw(LaunchError, "Launch Failed: " << ex.what());
     } catch (const std::exception& ex) {
         LOG_FATAL(dctl_logger, DCTL_PID_FILE_ERROR)
                   .arg(app_name_).arg(ex.what());
-        isc_throw (LaunchError, "Launch failed: " << ex.what());
+        isc_throw(LaunchError, "Launch failed: " << ex.what());
     }
 
     // Log the starting of the service.
@@ -135,8 +136,8 @@ DControllerBase::launch(int argc, char* argv[], const bool test_mode) {
     } catch (const std::exception& ex) {
         LOG_FATAL(dctl_logger, DCTL_INIT_PROCESS_FAIL)
                   .arg(app_name_).arg(ex.what());
-        isc_throw (ProcessInitError,
-                   "Application Process initialization failed: " << ex.what());
+        isc_throw(ProcessInitError,
+                  "Application Process initialization failed: " << ex.what());
     }
 
     LOG_DEBUG(dctl_logger, isc::log::DBGLVL_START_SHUT, DCTL_STANDALONE)
@@ -148,8 +149,8 @@ DControllerBase::launch(int argc, char* argv[], const bool test_mode) {
     if (rcode != 0) {
         LOG_FATAL(dctl_logger, DCTL_CONFIG_FILE_LOAD_FAIL)
                   .arg(app_name_).arg(comment->stringValue());
-        isc_throw (ProcessInitError, "Could Not load configuration file: "
-                   << comment->stringValue());
+        isc_throw(ProcessInitError, "Could Not load configuration file: "
+                  << comment->stringValue());
     }
 
     // Note that the controller was started.
@@ -164,8 +165,8 @@ DControllerBase::launch(int argc, char* argv[], const bool test_mode) {
     } catch (const std::exception& ex) {
         LOG_FATAL(dctl_logger, DCTL_PROCESS_FAILED)
                   .arg(app_name_).arg(ex.what());
-        isc_throw (ProcessRunError,
-                   "Application process event loop failed: " << ex.what());
+        isc_throw(ProcessRunError,
+                  "Application process event loop failed: " << ex.what());
     }
 
     // All done, so bail out.
@@ -250,9 +251,14 @@ DControllerBase::parseArgs(int argc, char* argv[]) {
     // ("c" or "d") handle it here.  If its a valid custom option, then
     // invoke customOption.
     int ch;
+    optarg = 0;
     opterr = 0;
     optind = 1;
     std::string opts("dvVWc:t:" + getCustomOpts());
+
+    // Defer exhausting of arguments to the end.
+    ExhaustOptions e(argc, argv, opts);
+
     while ((ch = getopt(argc, argv, opts.c_str())) != -1) {
         switch (ch) {
         case 'd':
@@ -293,10 +299,12 @@ DControllerBase::parseArgs(int argc, char* argv[]) {
             break;
 
         case '?': {
+            char const saved_optopt(optopt);
+            std::string const saved_optarg(optarg ? optarg : std::string());
+
             // We hit an invalid option.
-            isc_throw(InvalidUsage, "unsupported option: ["
-                      << static_cast<char>(optopt) << "] "
-                      << (!optarg ? "" : optarg));
+            isc_throw(InvalidUsage, "unsupported option: -" << saved_optopt <<
+                      (saved_optarg.empty() ? std::string() : " " + saved_optarg));
 
             break;
             }
@@ -304,10 +312,12 @@ DControllerBase::parseArgs(int argc, char* argv[]) {
         default:
             // We hit a valid custom option
             if (!customOption(ch, optarg)) {
-                // This would be a programmatic error.
-                isc_throw(InvalidUsage, " Option listed but implemented?: ["
-                          << static_cast<char>(ch) << "] "
-                          << (!optarg ? "" : optarg));
+                char const saved_optopt(optopt);
+                std::string const saved_optarg(optarg ? optarg : std::string());
+
+                // We hit an invalid option.
+                isc_throw(InvalidUsage, "unsupported option: -" << saved_optopt <<
+                          (saved_optarg.empty() ? std::string() : " " + saved_optarg));
             }
             break;
         }
@@ -334,8 +344,8 @@ DControllerBase::initProcess() {
     try {
         process_.reset(createProcess());
     } catch (const std::exception& ex) {
-        isc_throw(DControllerBaseError, std::string("createProcess failed: ")
-                  + ex.what());
+        isc_throw(DControllerBaseError, std::string("createProcess failed: ") +
+                  ex.what());
     }
 
     // This is pretty unlikely, but will test for it just to be safe..
@@ -530,7 +540,7 @@ DControllerBase::handleOtherObjects(ConstElementPtr args) {
     // Check obsolete or unknown (aka unsupported) objects.
     const std::string& app_name = getAppName();
     std::string errmsg;
-    for (auto obj : args->mapValue()) {
+    for (auto const& obj : args->mapValue()) {
         const std::string& obj_name = obj.first;
         if (obj_name == app_name) {
             continue;
@@ -725,7 +735,7 @@ DControllerBase::shutdownHandler(const std::string&, ConstElementPtr args) {
         }
 
         ConstElementPtr param = args->get("exit-value");
-        if (param)  {
+        if (param) {
             if (param->getType() != Element::integer) {
                 return (createAnswer(CONTROL_RESULT_ERROR,
                                      "parameter 'exit-value' is not an integer"));
@@ -805,7 +815,7 @@ DControllerBase::usage(const std::string & text) {
         std::cerr << "Usage error: " << text << std::endl;
     }
 
-    std::cerr << "Usage: " << bin_name_ <<  std::endl
+    std::cerr << "Usage: " << bin_name_ << std::endl
               << "  -v: print version number and exit" << std::endl
               << "  -V: print extended version information and exit"
               << std::endl
@@ -836,8 +846,15 @@ DControllerBase::~DControllerBase() {
         LOG_ERROR(dctl_logger, DCTL_UNLOAD_LIBRARIES_ERROR).arg(msg);
     }
 
+    IOServiceMgr::instance().clearIOServices();
+
     io_signal_set_.reset();
-    getIOService()->poll();
+    try {
+        getIOService()->poll();
+    } catch (...) {
+        // Don't want to throw exceptions from the destructor. The process
+        // is shutting down anyway.
+    }
 }
 
 std::string
@@ -846,10 +863,11 @@ DControllerBase::getVersion(bool extended) {
 
     tmp << VERSION;
     if (extended) {
-        tmp << std::endl << EXTENDED_VERSION << std::endl;
+        tmp << " (" << EXTENDED_VERSION << ")" << std::endl;
+        tmp << "premium: " << PREMIUM_EXTENDED_VERSION << std::endl;
         tmp << "linked with:" << std::endl;
-        tmp << isc::log::Logger::getVersion() << std::endl;
-        tmp << getVersionAddendum();
+        tmp << "- " << isc::log::Logger::getVersion() << std::endl;
+        tmp << "- " << isc::cryptolink::CryptoLink::getVersion();
     }
 
     return (tmp.str());
