@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,7 +7,6 @@
 #include <config.h>
 #include <cc/data.h>
 #include <process/log_parser.h>
-#include <boost/foreach.hpp>
 #include <boost/lexical_cast.hpp>
 #include <log/logger_specification.h>
 #include <log/logger_support.h>
@@ -33,7 +32,7 @@ void LogConfigParser::parseConfiguration(const isc::data::ConstElementPtr& logge
     verbose_ = verbose;
 
     // Iterate over all entries in "Server/loggers" list
-    BOOST_FOREACH(ConstElementPtr logger, loggers->listValue()) {
+    for (auto const& logger : loggers->listValue()) {
         parseConfigEntry(logger);
     }
 }
@@ -104,7 +103,7 @@ void LogConfigParser::parseConfigEntry(isc::data::ConstElementPtr entry) {
         info.debuglevel_ = 99;
     }
 
-    isc::data::ConstElementPtr output_options = entry->get("output_options");
+    isc::data::ConstElementPtr output_options = entry->get("output-options");
 
     if (output_options) {
         parseOutputOptions(info.destinations_, output_options);
@@ -116,16 +115,16 @@ void LogConfigParser::parseConfigEntry(isc::data::ConstElementPtr entry) {
 void LogConfigParser::parseOutputOptions(std::vector<LoggingDestination>& destination,
                                          isc::data::ConstElementPtr output_options) {
     if (!output_options) {
-        isc_throw(BadValue, "Missing 'output_options' structure in 'loggers'");
+        isc_throw(BadValue, "Missing 'output-options' structure in 'loggers'");
     }
 
-    BOOST_FOREACH(ConstElementPtr output_option, output_options->listValue()) {
+    for (auto const& output_option : output_options->listValue()) {
 
         LoggingDestination dest;
 
         isc::data::ConstElementPtr output = output_option->get("output");
         if (!output) {
-            isc_throw(BadValue, "output_options entry does not have a mandatory 'output' "
+            isc_throw(BadValue, "output-options entry does not have a mandatory 'output' "
                       "element (" << output_option->getPosition() << ")");
         }
         dest.output_ = output->stringValue();
