@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -42,7 +42,7 @@ TEST(ConfigCtlInfoParser, validConfigs) {
        "] } \n"
     };
 
-    for (auto config : configs) {
+    for (auto const& config : configs) {
         ConfigControlParser parser;
         ConfigControlInfoPtr ctl_info;
 
@@ -87,7 +87,7 @@ TEST(ConfigCtlInfoParser, invalidConfigs) {
        "{ \"config-fetch-wait-time\": \"a-string\" }",
     };
 
-    for (auto config : configs) {
+    for (auto const& config : configs) {
         ConfigControlParser parser;
 
         // Turn the JSON config into Elements.
