@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,7 +12,6 @@
 #include <process/d_cfg_mgr.h>
 #include <process/redact_config.h>
 
-#include <boost/foreach.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <gtest/gtest.h>
 
@@ -60,7 +59,7 @@ class DStubCfgMgrTest : public ConfigParseTest {
 public:
 
     /// @brief Constructor
-    DStubCfgMgrTest():cfg_mgr_(new DStubCfgMgr) {
+    DStubCfgMgrTest():cfg_mgr_(new DStubCfgMgr()) {
     }
 
     /// @brief Destructor
