@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,11 +6,12 @@
 
 #include <config.h>
 
+#include <gtest/gtest.h>
+
+#include <yang/tests/sysrepo_setup.h>
 #include <yang/translator_class.h>
 #include <yang/yang_models.h>
-#include <yang/tests/sysrepo_setup.h>
 
-#include <gtest/gtest.h>
 #include <sstream>
 
 using namespace std;
@@ -29,21 +30,20 @@ extern char const client_classes[] = "client classes";
 class TranslatorClassesTestv4 :
     public GenericTranslatorTest<client_classes, TranslatorClasses> {
 public:
-
-    /// Constructor.
+    /// @brief Constructor
     TranslatorClassesTestv4() {
         model_ = KEA_DHCP4_SERVER;
     }
-};
+};  // TranslatorClassesTestv4
+
 class TranslatorClassesTestv6 :
     public GenericTranslatorTest<client_classes, TranslatorClasses> {
 public:
-
-    /// Constructor.
+    /// @brief Constructor
     TranslatorClassesTestv6() {
         model_ = KEA_DHCP6_SERVER;
     }
-};
+};  // TranslatorClassesTestv6
 
 // This test verifies that an empty client class list can be properly
 // translated from YANG to JSON.
@@ -51,7 +51,7 @@ TEST_F(TranslatorClassesTestv4, getEmpty) {
     // Get the client class list and check if it is empty.
     const string& xpath = "/kea-dhcp4-server:config";
     ConstElementPtr classes;
-    EXPECT_NO_THROW(classes = t_obj_->getClasses(xpath));
+    EXPECT_NO_THROW_LOG(classes = translator_->getClassesFromAbsoluteXpath(xpath));
     EXPECT_FALSE(classes);
 }
 
@@ -62,21 +62,22 @@ TEST_F(TranslatorClassesTestv6, get) {
     const string& xpath = "/kea-dhcp6-server:config";
     const string& xclass = xpath + "/client-class[name='foo']";
     const string& xtest = xclass + "/test";
-    S_Val v_test(new Val("not member('ALL')", SR_STRING_T));
-    EXPECT_NO_THROW(sess_->set_item(xtest.c_str(), v_test));
+    string const v_test("not member('ALL')");
+    EXPECT_NO_THROW_LOG(sess_->setItem(xtest, v_test));
+    sess_->applyChanges();
 
     // Get the client class.
     ConstElementPtr cclass;
-    EXPECT_NO_THROW(cclass = t_obj_->getClass(xclass));
+    EXPECT_NO_THROW_LOG(cclass = translator_->getClassFromAbsoluteXpath(xclass));
     ASSERT_TRUE(cclass);
     ElementPtr expected = Element::createMap();
-    expected->set("name", Element::create(string("foo")));
-    expected->set("test", Element::create(string("not member('ALL')")));
+    expected->set("name", Element::create("foo"));
+    expected->set("test", Element::create("not member('ALL')"));
     EXPECT_TRUE(expected->equals(*cclass));
 
     // Get the client class list and check if the client class is in it.
     ConstElementPtr classes;
-    EXPECT_NO_THROW(classes = t_obj_->getClasses(xpath));
+    EXPECT_NO_THROW_LOG(classes = translator_->getClassesFromAbsoluteXpath(xpath));
     ASSERT_TRUE(classes);
     ASSERT_EQ(Element::list, classes->getType());
     ASSERT_EQ(1, classes->size());
@@ -89,11 +90,11 @@ TEST_F(TranslatorClassesTestv4, setEmpty) {
     // Set empty list.
     const string& xpath = "/kea-dhcp4-server:config";
     ConstElementPtr classes = Element::createList();
-    EXPECT_NO_THROW(t_obj_->setClasses(xpath, classes));
+    EXPECT_NO_THROW_LOG(translator_->setClasses(xpath, classes));
 
     // Get it back.
     classes.reset();
-    EXPECT_NO_THROW(classes = t_obj_->getClasses(xpath));
+    EXPECT_NO_THROW_LOG(classes = translator_->getClassesFromAbsoluteXpath(xpath));
     EXPECT_FALSE(classes);
 }
 
@@ -104,22 +105,19 @@ TEST_F(TranslatorClassesTestv6, set) {
     const string& xpath = "/kea-dhcp6-server:config";
     ElementPtr classes = Element::createList();
     ElementPtr cclass = Element::createMap();
-    cclass->set("name", Element::create(string("foo")));
-    cclass->set("test", Element::create(string("''==''")));
+    cclass->set("name", Element::create("foo"));
+    cclass->set("test", Element::create("''==''"));
     cclass->set("only-if-required", Element::create(false));
     classes->add(cclass);
-    EXPECT_NO_THROW(t_obj_->setClasses(xpath, classes));
+    EXPECT_NO_THROW_LOG(translator_->setClasses(xpath, classes));
 
     // Get it back.
     ConstElementPtr got;
-    EXPECT_NO_THROW(got = t_obj_->getClasses(xpath));
+    EXPECT_NO_THROW_LOG(got = translator_->getClassesFromAbsoluteXpath(xpath));
     ASSERT_TRUE(got);
     ASSERT_EQ(Element::list, got->getType());
     ASSERT_EQ(1, got->size());
     EXPECT_TRUE(cclass->equals(*got->get(0)));
-
-    // Check it validates.
-    EXPECT_NO_THROW(sess_->validate());
 }
 
 }  // namespace
